﻿using System;
using System.Text;
using jp.veritrans.tercerog.mdk;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;


namespace ConsoleApplication
{
    class Program
    {
        static void Main(string[] args)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

            using var servicesProvider = new ServiceCollection()
                .AddTransient<Runner>()
                .AddLogging(loggingBuilder =>
                {
                    loggingBuilder.ClearProviders();
                    loggingBuilder.SetMinimumLevel(LogLevel.Information);
                    loggingBuilder.AddNLog();
                }).BuildServiceProvider();

            var runner = servicesProvider.GetRequiredService<Runner>();
            runner.Run(args);
        }
    }

    public class Runner
    {
        private readonly ILogger<Runner> _logger;

        public Runner(ILogger<Runner> logger)
        {
            _logger = logger;
        }

        public void Run(string[] args)
        {
            var merchantConfig = new MerchantConfig(logger: _logger, merchantCcId: "{Set Merchant CcId here}",
                merchantSecret: "{Set Merchant Secret Key here}", true);

            if (args.Length == 1)
            {
                switch (args[0])
                {
                    case "alipay-authorize":
                        var alipayauthorize =
                            new alipay.CLI_AlipayAuthorize();
                        alipayauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "alipay-confirm":
                        var alipayconfirm =
                            new alipay.CLI_AlipayConfirm();
                        alipayconfirm.Exec(_logger, merchantConfig);
                        break;
                    case "alipay-refund":
                        var alipayrefund =
                            new alipay.CLI_AlipayRefund();
                        alipayrefund.Exec(_logger, merchantConfig);
                        break;

                    case "bank-authorize":
                        var bankauthorize =
                            new bank.CLI_BankAuthorize();
                        bankauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "card-authorize":
                        var cardauthorize =
                            new card.CLI_CardAuthorize();
                        cardauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "card-capture":
                        var cardcapture =
                            new card.CLI_CardCapture();
                        cardcapture.Exec(_logger, merchantConfig);
                        break;
                    case "card-cancel":
                        var cardcancel =
                            new card.CLI_CardCancel();
                        cardcancel.Exec(_logger, merchantConfig);
                        break;
                    case "card-reauthorize":
                        var cardreauthorize =
                            new card.CLI_CardReAuthorize();
                        cardreauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "cvs-authorize":
                        var cvsauthorize =
                            new cvs.CLI_CvsAuthorize();
                        cvsauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "cvs-cancel":
                        var cvscancel =
                            new cvs.CLI_CvsCancel();
                        cvscancel.Exec(_logger, merchantConfig);
                        break;
                    case "em-authorize":
                        var emauthorize =
                            new em.CLI_EmAuthorize();
                        emauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "em-reauthorize":
                        var emreauthorize =
                            new em.CLI_EmReAuthorize();
                        emreauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "em-cancel":
                        var emcancel =
                            new em.CLI_EmCancel();
                        emcancel.Exec(_logger, merchantConfig);
                        break;
                    case "em-refund":
                        var emrefund =
                            new em.CLI_EmRefund();
                        emrefund.Exec(_logger, merchantConfig);
                        break;
                    case "em-remove":
                        var emremove =
                            new em.CLI_EmRemove();
                        emremove.Exec(_logger, merchantConfig);
                        break;
                    case "mpi-authorize":
                        var mpiauthorize =
                            new mpi.CLI_MpiAuthorize();
                        mpiauthorize.Exec(_logger, merchantConfig);
                        break;
                    case "mpi-getresult":
                        var mpigetresult =
                            new mpi.CLI_MpiGetResult();
                        mpigetresult.Exec(_logger, merchantConfig);
                        break;
                    case "mpi-reauthorize":
                        var mpireauthorize =
                            new mpi.CLI_MpiReAuthorize();
                        mpireauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "saison-capture":
                        var saisoncapture =
                            new saison.CLI_SaisonCapture();
                        saisoncapture.Exec(_logger, merchantConfig);
                        break;

                    case "saison-cancel":
                        var saisoncancel =
                            new saison.CLI_SaisonCancel();
                        saisoncancel.Exec(_logger, merchantConfig);
                        break;

                    case "carrier-authorize":
                        var carrierauthorize =
                            new carrier.CLI_CarrierAuthorize();
                        carrierauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "carrier-cancel":
                        var carriercancel =
                            new carrier.CLI_CarrierCancel();
                        carriercancel.Exec(_logger, merchantConfig);
                        break;

                    case "carrier-capture":
                        var carriercapture =
                            new carrier.CLI_CarrierCapture();
                        carriercapture.Exec(_logger, merchantConfig);
                        break;

                    case "carrier-terminate":
                        var carrierterminate =
                            new carrier.CLI_CarrierTerminate();
                        carrierterminate.Exec(_logger, merchantConfig);
                        break;

                    case "carrier-reauthorize":
                        var carrierreauthorize =
                            new carrier.CLI_CarrierReAuthorize();
                        carrierreauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "rakuten-authorize":
                        var rakutenauthorize =
                            new rakuten.CLI_RakutenAuthorize();
                        rakutenauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "rakuten-capture":
                        var rakutencapture =
                            new rakuten.CLI_RakutenCapture();
                        rakutencapture.Exec(_logger, merchantConfig);
                        break;

                    case "rakuten-cancel":
                        var rakutencancel =
                            new rakuten.CLI_RakutenCancel();
                        rakutencancel.Exec(_logger, merchantConfig);
                        break;

                    case "rakuten-updateauthorize":
                        var rakutenupdateauthorize =
                            new rakuten.CLI_RakutenUpdateAuthorize();
                        rakutenupdateauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "rakuten-reauthorize":
                        var rakutenreauthorize =
                            new rakuten.CLI_RakutenReAuthorize();
                        rakutenreauthorize.Exec(_logger, merchantConfig);
                        break;

                    case "recruit-authorize":
                        var recruitAuthorize =
                            new recruit.CLI_RecruitAuthorize();
                        recruitAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "recruit-capture":
                        var recruitCapture =
                            new recruit.CLI_RecruitCapture();
                        recruitCapture.Exec(_logger, merchantConfig);
                        break;

                    case "recruit-cancel":
                        var recruitCancel =
                            new recruit.CLI_RecruitCancel();
                        recruitCancel.Exec(_logger, merchantConfig);
                        break;

                    case "recruit-extendauth":
                        var recruitExtendAuth =
                            new recruit.CLI_RecruitExtendAuth();
                        recruitExtendAuth.Exec(_logger, merchantConfig);
                        break;

                    case "linepay-authorize":
                        var linepayAuthorize =
                            new linepay.CLI_LinepayAuthorize();
                        linepayAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "linepay-capture":
                        var linepayCapture =
                            new linepay.CLI_LinepayCapture();
                        linepayCapture.Exec(_logger, merchantConfig);
                        break;

                    case "linepay-cancel":
                        var linepayCancel =
                            new linepay.CLI_LinepayCancel();
                        linepayCancel.Exec(_logger, merchantConfig);
                        break;

                    case "masterpass-authorize":
                        var masterpassAuthorize =
                            new masterpass.CLI_MasterpassAuthorize();
                        masterpassAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "masterpass-capture":
                        var masterpassCapture =
                            new masterpass.CLI_MasterpassCapture();
                        masterpassCapture.Exec(_logger, merchantConfig);
                        break;

                    case "masterpass-cancel":
                        var masterpassCancel =
                            new masterpass.CLI_MasterpassCancel();
                        masterpassCancel.Exec(_logger, merchantConfig);
                        break;

                    case "masterpass-login":
                        var masterpassLogin =
                            new masterpass.CLI_MasterpassLogin();
                        masterpassLogin.Exec(_logger, merchantConfig);
                        break;

                    case "upop-authorize":
                        var upopAuthorize =
                            new upop.CLI_UpopAuthorize();
                        upopAuthorize.Exec(_logger, merchantConfig);
                        break;
                    case "upop-capture":
                        var upopCapture =
                            new upop.CLI_UpopCapture();
                        upopCapture.Exec(_logger, merchantConfig);
                        break;
                    case "upop-cancel":
                        var upopCancel =
                            new upop.CLI_UpopCancel();
                        upopCancel.Exec(_logger, merchantConfig);
                        break;
                    case "upop-refund":
                        var upopRefund =
                            new upop.CLI_UpopRefund();
                        upopRefund.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-authorize":
                        var virtualaccAuthorize =
                            new virtualacc.CLI_VirtualaccAuthorize();
                        virtualaccAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-cancel":
                        var virtualaccCancel =
                            new virtualacc.CLI_VirtualaccCancel();
                        virtualaccCancel.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-depositentry":
                        var virtualaccDepositEntry =
                            new virtualacc.CLI_VirtualaccDepositEntry();
                        virtualaccDepositEntry.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-depositreverse":
                        var virtualaccDepositReverse =
                            new virtualacc.CLI_VirtualaccDepositReverse();
                        virtualaccDepositReverse.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-reconcile":
                        var virtualaccReconcile =
                            new virtualacc.CLI_VirtualaccReconcile();
                        virtualaccReconcile.Exec(_logger, merchantConfig);
                        break;

                    case "virtualacc-undoreconcile":
                        var virtualaccUndoReconcile =
                            new virtualacc.CLI_VirtualaccUndoReconcile();
                        virtualaccUndoReconcile.Exec(_logger, merchantConfig);
                        break;

                    case "paypay-online-authorize":
                        var paypayOnlineAuthorize = new paypay.CLI_PaypayOnlineAuthorize();
                        paypayOnlineAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "paypay-online-cancel":
                        var paypayOnlineCancel = new paypay.CLI_PaypayOnlineCancel();
                        paypayOnlineCancel.Exec(_logger, merchantConfig);
                        break;

                    case "paypay-online-capture":
                        var paypayOnlineCapture = new paypay.CLI_PaypayOnlineCapture();
                        paypayOnlineCapture.Exec(_logger, merchantConfig);
                        break;

                    case "paypay-online-refund":
                        var paypayOnlineRefund = new paypay.CLI_PaypayOnlineRefund();
                        paypayOnlineRefund.Exec(_logger, merchantConfig);
                        break;

                    case "amazonpay-authorize":
                        var amazonpayAuthorize = new amazonpay.CLI_AmazonpayAuthorize();
                        amazonpayAuthorize.Exec(_logger, merchantConfig);
                        break;

                    case "amazonpay-cancel":
                        var amazonpayCancel = new amazonpay.CLI_AmazonpayCancel();
                        amazonpayCancel.Exec(_logger, merchantConfig);
                        break;

                    case "amazonpay-capture":
                        var amazonpayCapture = new amazonpay.CLI_AmazonpayCapture();
                        amazonpayCapture.Exec(_logger, merchantConfig);
                        break;

                    case "amazonpay-getaddress":
                        var amazonpayGetAddress = new amazonpay.CLI_AmazonpayGetAddress();
                        amazonpayGetAddress.Exec(_logger, merchantConfig);
                        break;

                    case "amazonpay-refund":
                        var amazonpayRefund = new amazonpay.CLI_AmazonpayRefund();
                        amazonpayRefund.Exec(_logger, merchantConfig);
                        break;

                    case "search":
                        var search = new search.CLI_Search();
                        search.Exec(_logger, merchantConfig);
                        break;

                    case "account-add":
                        var accountAdd = new paynowid.CLI_PayNowIdAccountAddCommand();
                        accountAdd.Exec(_logger, merchantConfig);
                        break;

                    case "account-delete":
                        var accountDelete =
                            new paynowid.CLI_PayNowIdAccountDeleteCommand();
                        accountDelete.Exec(_logger, merchantConfig);
                        break;

                    case "account-restore":
                        var accountRestore =
                            new paynowid.CLI_PayNowIdAccountRestoreCommand();
                        accountRestore.Exec(_logger, merchantConfig);
                        break;

                    case "account-link":
                        var accountLink =
                            new paynowid.CLI_PayNowIdAccountLinkCommand();
                        accountLink.Exec(_logger, merchantConfig);
                        break;
                    case "card-add":
                        var cardAdd =
                            new paynowid.CLI_PayNowIdCardAddCommand();
                        cardAdd.Exec(_logger, merchantConfig);
                        break;
                    case "card-delete":
                        var cardDelete =
                            new paynowid.CLI_PayNowIdCardDeleteCommand();
                        cardDelete.Exec(_logger, merchantConfig);
                        break;
                    case "card-get":
                        var cardGet =
                            new paynowid.CLI_PayNowIdCardGetCommand();
                        cardGet.Exec(_logger, merchantConfig);
                        break;
                    case "card-update":
                        var cardUpdate =
                            new paynowid.CLI_PayNowIdCardUpdateCommand();
                        cardUpdate.Exec(_logger, merchantConfig);
                        break;
                    case "entry-and-card":
                        var entryAndCard =
                            new paynowid.CLI_PayNowIdEntryAndCardCommand();
                        entryAndCard.Exec(_logger, merchantConfig);
                        break;
                    case "entry-and-mpi":
                        var entryAndMpi = new paynowid.CLI_PayNowIdEntryAndMpiCommand();
                        entryAndMpi.Exec(_logger, merchantConfig);
                        break;
                    case "recurring-add":
                        var recurringAdd =
                            new paynowid.CLI_PayNowIdRecurringAddCommand();
                        recurringAdd.Exec(_logger, merchantConfig);
                        break;
                    case "recurring-delete":
                        var recurringDelete =
                            new paynowid.CLI_PayNowIdRecurringDeleteCommand();
                        recurringDelete.Exec(_logger, merchantConfig);
                        break;
                    case "recurring-update":
                        var recurringUpdate =
                            new paynowid.CLI_PayNowIdRecurringUpdateCommand();
                        recurringUpdate.Exec(_logger, merchantConfig);
                        break;

                    default:
                        Console.WriteLine("指定が正しくありません。");
                        break;
                }
            }
            else
            {
                Console.WriteLine("指定が正しくありません。");
            }
        }
    }
}