﻿using System;
using System.Collections.Generic;
using System.Text;
using jp.veritrans.tercerog.mdk.dto;
using jp.veritrans.tercerog.mdk;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.carrier
{
    /// <summary>
    /// キャリア決済(与信)サンプル
    /// </summary>
    class CLI_CarrierAuthorize
    {
        public virtual void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            /**
             * サービスオプションタイプ 
             * au          : auかんたん決済
             * docomo      : ドコモケータイ払い
             * sb_ktai     : ソフトバンクまとめて支払い(B)
             * sb_matomete : ソフトバンクまとめて支払い(A)
             */
            String serviceOptionType = "au";
            /**
             * 取引ID
             */
            String orderId = "dummy" + System.DateTime.Now.ToString("HHmmssfff");
            /**
             * 与信方法
             * ソフトバンクまとめて支払い（A）は与信同時売上のみのため "true" を
             * 指定する必要があります
             */
            String withCapture = "false";
            /**
             * 金額
             *
             * ドコモ随時決済の場合、金額は指定できません
             */
            String amount = "10";
            /**
             * 商品番号
             */
            String itemId = "12345";
            /**
             * 課金種別
             * 0 : 都度
             * 1 : 継続
             * 4 : 随時
             *
             * 随時はドコモのみ指定可能
             */
            String accountingType = "0";
            /**
             * 端末種別
             * 0 : PC
             * 1 : スマートフォン
             * 2 : フィーチャーフォン
             */
            String terminalKind = "0";
            /**
             * 商品タイプ
             * 0 : デジタルコンテンツ
             * 1 : 物販
             * 2 : 役務
             */
            String itemType = "0";
            /**
             * OpenID
             */
            String openId = "";

            CarrierAuthorizeRequestDto requestData = new CarrierAuthorizeRequestDto();
            requestData.ServiceOptionType = serviceOptionType;
            requestData.OrderId = orderId;
            requestData.WithCapture = withCapture;
            requestData.Amount = amount;
            requestData.ItemId = itemId;
            requestData.AccountingType = accountingType;
            requestData.TerminalKind = terminalKind;
            requestData.ItemType = itemType;
            requestData.OpenId = openId;

            /**
             * 実行と結果表示  
            */
            Transaction tran = new Transaction(logger, merchantConfig);
            CarrierAuthorizeResponseDto responseDate = (CarrierAuthorizeResponseDto)tran.Execute(requestData);

            System.Console.WriteLine("<< REQUEST >>");
            System.Console.WriteLine("Order ID:" + requestData.OrderId);
            System.Console.WriteLine("Service Option Type:" + requestData.ServiceOptionType);
            System.Console.WriteLine("Amount:" + requestData.Amount);
            System.Console.WriteLine("With Capture:" + requestData.WithCapture);
            System.Console.WriteLine("Accounting Type:" + requestData.AccountingType);
            System.Console.WriteLine("Terminal Kind:" + requestData.TerminalKind);
            System.Console.WriteLine("Item Type:" + requestData.ItemType);

            System.Console.WriteLine("<< RESPONSE >>");
            System.Console.WriteLine("MStatus:" + responseDate.Mstatus);
            System.Console.WriteLine("Message:" + responseDate.MerrMsg);
            System.Console.WriteLine("Result Code:" + responseDate.VResultCode);
            System.Console.WriteLine("Response Contents:" + responseDate.ResponseContents);
        }
    }
}
