﻿using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.em
{
    /// <summary>
    /// 電子マネー決済 取消要求サンプル
    /// </summary>
    public partial class CLI_EmCancel
    {
        public virtual void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            /**
             * 決済方式
             *   
             * モバイルSuicaメール決済　　　　　 : suica-mobile-mail
             * モバイルSuicaアプリ決済 　　　　　: suica-mobile-app
             * Suicaインターネット決済メール連携 : suica-pc-mail
             * Suicaインターネット決済アプリ連携 : suica-pc-app
             * nanaco決済                        : tcc-redirect
             * BitCash決済                       : bitcash-api
             */
            String serviceOptionType = "suica-mobile-mail";

            /**
             * 取引ID
             * 申込時の取引IDを指定
             */
            String orderId = "";
            /**
             * オーダー種別
             */
            String orderKind = "authorize";
            /**
             * 取消通知用メールアドレス
             */
            String cancelMailAddress = "dummy-support@vt-mdk.co.jp";

            /**
             * 要求電文パラメータ値の指定
             */
            EmCancelRequestDto requestData = new EmCancelRequestDto();
            requestData.ServiceOptionType = serviceOptionType;
            requestData.OrderId = orderId;

            if (serviceOptionType != "tcc-redirect" && serviceOptionType != "bitcash-api" )
            {
                requestData.OrderKind = orderKind;
                requestData.CancelMailAddr = cancelMailAddress;
            }

            /**
             * 実行と結果表示
             */
            Transaction tran = new Transaction(logger, merchantConfig);
            EmCancelResponseDto responseData = (EmCancelResponseDto)tran.Execute(requestData);

            System.Console.WriteLine("<< REQUEST >>");
            System.Console.WriteLine("ServiceOptionType:" + requestData.ServiceOptionType);
            System.Console.WriteLine("Order ID:" + requestData.OrderId);

            System.Console.WriteLine("<< RESPONSE >>");
            System.Console.WriteLine("MStatus:" + responseData.Mstatus);
            System.Console.WriteLine("Message:" + responseData.MerrMsg);
            System.Console.WriteLine("Result Code:" + responseData.VResultCode);
            System.Console.WriteLine("Log Id:" + responseData.LogId);
            System.Console.WriteLine("Card Balance:" + responseData.CardBalance);
            System.Console.WriteLine("Center Request Datetime:" + responseData.CenterRequestDatetime);
            System.Console.WriteLine("Center Response Datetime:" + responseData.CenterResponseDatetime);
        }
    }
}
