﻿using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.em
{
    /// <summary>
    /// 電子マネー決済　返金要求サンプル
    /// </summary>
    public partial class CLI_EmRefund
    {
        public virtual void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            /**
             * 決済方式
             * 
             * サイバー Edy     (パソリ)         : edy-pc
             * モバイル Edy                      : edy-mobile
             * モバイル Edy ダイレクト           : edy-direct
             * モバイルSuicaメール決済           : suica-mobile-mail
             * モバイルSuicaアプリ決済           : suica-mobile-app
             * Suicaインターネット決済メール連携 : suica-pc-mail
             * Suicaインターネット決済アプリ連携 : suica-pc-app
             */
            String serviceOptionType = "suica-mobile-mail";

            /**
             * 取引ID
             */
            String orderId = "dummy" + System.DateTime.Now.ToString("HHmmssfff");
            /**
             * 返金依頼金額  <= 決済金額
             */
            String amount = "100";
            /**
             * オーダー種別
             * 返金    ：refund
             * 新規返金 : refund_new
             */
            String orderKind = "refund";
            /**
             * 返金対象取引ID
             */
            String refundOrderId = "dummy_target_id";
            /**
             * 決済期限
             * Edyでは利用しません。
             */
            String settlementLimit = System.DateTime.Now.AddDays(5).ToString("yyyyMMddHHmmss");
            /**
             * メールアドレス
             * Cyber Edy、Suica決済アプリ連携未使用
             */
            String requesterMailAddress = "sample.sample@vt-mdk.co.jp";

            /**
             * 取消通知用メールアドレス
             */
            String cancelMailAddress = "dummy-support@vt-mdk.co.jp";
            /**
             * 画面タイトル 
             * Edyでは利用しません。
             */
            String screenTitle = "For Suica";

            /**
             * 要求電文パラメータ値の指定
             */
            EmRefundRequestDto requestData = new EmRefundRequestDto();
            requestData.ServiceOptionType = serviceOptionType;
            requestData.OrderId = orderId;
            requestData.RefundOrderId = refundOrderId;
            requestData.OrderKind = orderKind;
            requestData.Amount = amount;
            if (serviceOptionType.Equals("edy-pc") == false &&
                serviceOptionType.Equals("edy-mobile") == false &&
                serviceOptionType.Equals("edy-direct") == false)
            {
                requestData.SettlementLimit = settlementLimit;
            }
            if (serviceOptionType.Equals("suica-mobile-mail") == true ||
                serviceOptionType.Equals("suica-pc-mail") == true
                )
            {
                requestData.MailAddr = requesterMailAddress;
            }
            if (serviceOptionType.Equals("edy-pc") == false &&
                serviceOptionType.Equals("edy-mobile") == false &&
                serviceOptionType.Equals("edy-direct") == false)
            {
                requestData.ScreenTitle = screenTitle;
            }
            requestData.CancelMailAddr = cancelMailAddress;

            /**
             * 実行と結果表示
             */
            Transaction tran = new Transaction(logger, merchantConfig);
            EmRefundResponseDto responseData = (EmRefundResponseDto)tran.Execute(requestData);
        
            System.Console.WriteLine("<< REQUEST >>");
            System.Console.WriteLine("ServiceOptionType:" + requestData.ServiceOptionType);
            System.Console.WriteLine("Order ID:" + requestData.OrderId);
            System.Console.WriteLine("Amount:" + requestData.Amount);

            System.Console.WriteLine("<< RESPONSE >>");
            System.Console.WriteLine("MStatus:" + responseData.Mstatus);
            System.Console.WriteLine("Message:" + responseData.MerrMsg);
            System.Console.WriteLine("Result Code:" + responseData.VResultCode);
        }
    }
}
