﻿using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.mpi
{
    /// <summary>
    /// クレジットカード決済(本人認証) 再与信要求サンプル
    /// </summary>
    public partial class CLI_MpiReAuthorize
    {
        public virtual void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            /**
             * オプションタイプ
             * 1. MPIサービスのみ                         : mpi-none
             * 2. 完全認証                                : mpi-complete
             * 3. 通常認証（カード会社リスク負担）       ： mpi-company
             * 4. 通常認証（カード会社、加盟店リスク負担）: mpi-merchant
             * 
             */
            String serviceOptionType = "mpi-company";
            /**
             * 取引ID
             */
            String orderId = "dummy" + System.DateTime.Now.ToString("HHmmssfff");
            /**
             * 対象元取引ID
             */
            String originalOrderId = "";
            /**
             * 決済金額
             */
            String paymentAmount = "10";

            /**
             * HTTP User-Agent（消費者ブラウザ）
             * （実際はアプリケーションサーバから取得）
             */
            String httpUserAgent = "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)";
            /**
             * HTTP Accept（消費者ブラウザ）
             * （実際はアプリケーションサーバから取得）
             */
            String httpAccept = "text/*;q=0.3, text/html;q=0.7";
            /**
             * デバイスチャネル
             * 02:ブラウザベース
             */
            String deviceChannel = "02";
            /*
             * カード保有者名
             */
            String cardholderName = "cardholderName";

            /**
             * 要求電文パラメータ値の指定
             */
            MpiReAuthorizeRequestDto requestData = new MpiReAuthorizeRequestDto();
            requestData.ServiceOptionType = serviceOptionType;
            requestData.OrderId = orderId;
            requestData.OriginalOrderId = originalOrderId;
            requestData.Amount = paymentAmount;
            requestData.HttpUserAgent = httpUserAgent;
            requestData.HttpAccept = httpAccept;
            requestData.DeviceChannel = deviceChannel;
            requestData.CardholderName = cardholderName;

            /**
             * 実行と結果表示
             */
            Transaction tran = new Transaction(logger, merchantConfig);
            MpiReAuthorizeResponseDto responseData = (MpiReAuthorizeResponseDto)tran.Execute(requestData);

            System.Console.WriteLine("<< REQUEST >>");
            System.Console.WriteLine("Order ID:" + requestData.OrderId);
            System.Console.WriteLine("WithCapture:" + requestData.WithCapture);
            System.Console.WriteLine("Amount:" + requestData.Amount);
            System.Console.WriteLine("JPO:" + requestData.Jpo);
            System.Console.WriteLine("User-Agent:" + requestData.HttpUserAgent);
            System.Console.WriteLine("Accept:" + requestData.HttpAccept);
            System.Console.WriteLine("Device Channel:" + requestData.DeviceChannel);
            System.Console.WriteLine("Cardholder Name:" + requestData.CardholderName);

            System.Console.WriteLine("<< RESPONSE >>");
            System.Console.WriteLine("MStatus:" + responseData.Mstatus);
            System.Console.WriteLine("Message:" + responseData.MerrMsg);
            System.Console.WriteLine("Result Code:" + responseData.VResultCode);
            System.Console.WriteLine("Auth Start URL:" + responseData.AuthStartUrl);
        }
    }
}
