using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.paynowid
{
    /// <summary>
    /// PayNowId(会員復活)処理用サーブレット
    /// </summary>
    public class CLI_PayNowIdAccountRestoreCommand
    {
        /// <summary>
        /// コマンドテスト用メインメソッド
        /// </summary>
        public void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            // ---------------------------------
            // テスト用リクエスト電文項目設定
            // ---------------------------------
            // 会員ID
            String accountId = "TestAccountId";
            // 入会年月日
            String createDate = "20130930";

            // ---------------------------------
            // リクエスト DTO 設定
            // ---------------------------------
            var reqDto = new AccountRestoreRequestDto
                {
                    AccountId = accountId, // 会員IDの設定
                    CreateDate = createDate // 入会年月日
                };

            // ---------------------------------
            // 処理実行と結果表示
            // ---------------------------------

            try
            {
                var tran = new Transaction(logger, merchantConfig);
                var resDto = (AccountRestoreResponseDto) tran.Execute(reqDto);

                // リクエストとレスポンスを実行コンソール上で表示する
                Console.WriteLine("*- PayNowId(Restore) -*");

                Console.WriteLine("<< REQUEST >>");
                Console.WriteLine("ServiceType:"+  reqDto.ServiceType);
                Console.WriteLine("Account ID:" + accountId);
                Console.WriteLine("Create Date:" + createDate);

                Console.WriteLine("<< RESPONSE >>");
                Console.WriteLine("PayNowId Status:" + resDto.PayNowIdResponse.Status);
                Console.WriteLine("Message:" + resDto.MerrMsg);
                Console.WriteLine("Result Code:" + resDto.VResultCode);
                Console.WriteLine("PayNowId ProcId:" + resDto.PayNowIdResponse.ProcessId);

            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception Occured !");
                Console.WriteLine("Exception Message " + ex.Message);
            }

        }
    }
}