using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.paynowid
{
    /// <summary>PayNowId(カード情報登録)処理用サーブレット</summary>
    public class CLI_PayNowIdCardAddCommand
    {
        /// <summary>
        /// コマンドテスト用メインメソッド
        /// </summary>
        public void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            // ---------------------------------
            // テスト用リクエスト電文項目設定
            // ---------------------------------
            // 会員ID
            String accountId = "TestAccountId";
            // カード番号
            // String cardNumber = "4111-1111-1111-1111";
            // カード有効期限
            // String cardExpire = "12/16";
            // トークン
            String token = "0a812412-682c-4dad-8a5d-720caf23bca0";

            // ---------------------------------
            // リクエスト DTO 設定
            // ---------------------------------
            var reqDto = new CardInfoAddRequestDto
                {
                    AccountId = accountId, // 会員IDの設定
                    // CardNumber = cardNumber, // カード番号
                    // CardExpire = cardExpire // カード有効期限
                    Token = token // トークン
                };

            // ---------------------------------
            // 処理実行と結果表示
            // ---------------------------------

            try
            {
                var tran = new Transaction(logger, merchantConfig);
                var resDto = (CardInfoAddResponseDto)tran.Execute(reqDto);
                var payNowIdRes = resDto.PayNowIdResponse;

                // リクエストとレスポンスを実行コンソール上で表示する
                Console.WriteLine("*- PayNowId(Add) -*");

                Console.WriteLine("<< REQUEST >>");
                Console.WriteLine("ServiceType:" + reqDto.ServiceType);
                Console.WriteLine("Account ID:" + accountId);
                // Console.WriteLine("Card Number:" + cardNumber);
                // Console.WriteLine("Card Expired Date:" + cardExpire);
                Console.WriteLine("Token:" + token);

                Console.WriteLine("<< RESPONSE >>");
                Console.WriteLine("PayNowId Status:" + ((payNowIdRes != null) ? payNowIdRes.Status : "I do not know status because of PayNowIdResponse is null."));
                Console.WriteLine("Message:" + resDto.MerrMsg);
                Console.WriteLine("Result Code:" + resDto.VResultCode);
                Console.WriteLine("PayNowId ProcId:" + ((payNowIdRes != null) ? payNowIdRes.ProcessId : "I do not know processId because of PayNowIdResponse is null."));

            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception Occured !");
                Console.WriteLine("Exception Message " + ex.Message);
            }

        }
    }
}