using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.paynowid
{
    /// <summary>PayNowId(カード情報削除)処理用サーブレット</summary>
    public class CLI_PayNowIdCardDeleteCommand
    {
        /// <summary>コマンドテスト用メインメソッド</summary>
        public void Exec(ILogger logger, MerchantConfig merchantConfig)
        {
            // ---------------------------------
            // テスト用リクエスト電文項目設定
            // ---------------------------------
            // 会員ID
            String accountId = "TestAccountId";
            // カードID
            String cardId = "1";

            // ---------------------------------
            // リクエスト DTO 設定
            // ---------------------------------
            var reqDto = new CardInfoDeleteRequestDto
                {
                    AccountId = accountId, // 会員IDの設定
                    CardId = cardId // カードID
                };

            // ---------------------------------
            // 処理実行と結果表示
            // ---------------------------------

            try
            {
                var tran = new Transaction(logger, merchantConfig);
                var resDto = (CardInfoDeleteResponseDto)tran.Execute(reqDto);

                // リクエストとレスポンスを実行コンソール上で表示する
                Console.WriteLine("*- PayNowId(Delete) -*");

                Console.WriteLine("<< REQUEST >>");
                Console.WriteLine("ServiceType:" + reqDto.ServiceType);
                Console.WriteLine("Account ID:" + accountId);
                Console.WriteLine("Card ID:" + cardId);

                Console.WriteLine("<< RESPONSE >>");
                Console.WriteLine("PayNowId Status:" + resDto.PayNowIdResponse.Status);
                Console.WriteLine("Message:" + resDto.MerrMsg);
                Console.WriteLine("Result Code:" + resDto.VResultCode);
                Console.WriteLine("PayNowId ProcId:" + resDto.PayNowIdResponse.ProcessId);

            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception Occured !");
                Console.WriteLine("Exception Message " + ex.Message);
            }
        }
    }
}