using System;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.Extensions.Logging;

namespace ConsoleApplication.paynowid
{
    /// <summary>PayNowId(継続課金登録)処理用サーブレット</summary>
    public class CLI_PayNowIdRecurringAddCommand
    {

        /// <summary>コマンドテスト用メインメソッド</summary>
        public void Exec(ILogger logger, MerchantConfig merchantConfig)
        {

            // ---------------------------------
            // テスト用リクエスト電文項目設定
            // ---------------------------------
            // 会員ID
            String accountId = "TestAccountId";
            // 課金グループID設定
            String groupId = "merchant4";
            // 課金開始日設定
            String startDate = "20130930";
            // 課金終了日設定
            String endDate = "20140930";
            // 都度／初回課金金額設定
            String oneTimeAmount = "10";
            // 継続課金金額設定
            String amount = "10";
            // カード番号
            // String cardNumber = "4111-1111-1111-1111";
            // カード有効期限
            // String cardExpire = "12/16";
            // トークン
            String token = "0a812412-682c-4dad-8a5d-720caf23bca0";

            // ---------------------------------
            // リクエスト DTO 設定
            // ---------------------------------
            var reqDto = new RecurringAddRequestDto
                {
                    AccountId = accountId, // 会員ID設定
                    GroupId = groupId, // 課金グループID設定
                    StartDate = startDate, // 課金開始日設定
                    EndDate = endDate, // 課金終了日設定
                    OneTimeAmount = oneTimeAmount, // 都度／初回課金金額設定
                    Amount = amount, // 継続課金金額設定
                    // CardNumber = cardNumber, // カード番号設定
                    // CardExpire = cardExpire // 有効期限設定
                    Token = token // トークン設定
                };

            // ---------------------------------
            // 処理実行と結果表示
            // ---------------------------------

            try
            {
                var tran = new Transaction(logger, merchantConfig);
                var resDto = (RecurringAddResponseDto) tran.Execute(reqDto);
                var payNowIdRes = resDto.PayNowIdResponse;

                // リクエストとレスポンスを実行コンソール上で表示する
                Console.WriteLine("*- PayNowId(Add) -*");

                Console.WriteLine("<< REQUEST >>");
                Console.WriteLine("ServiceType:" + reqDto.ServiceType);
                Console.WriteLine("Account ID:" + accountId);
                Console.WriteLine("Group ID:" + groupId);
                Console.WriteLine("Start Date:" + startDate);
                Console.WriteLine("End Date:" + endDate);
                Console.WriteLine("OneTimeAmount:" + oneTimeAmount);
                Console.WriteLine("Amount:" + amount);
                // Console.WriteLine("Card Number:" + cardNumber);
                // Console.WriteLine("Card Expired Date:" + cardExpire);
                Console.WriteLine("Token:" + token);

                Console.WriteLine("<< RESPONSE >>");
                Console.WriteLine("PayNowId Status:" + ((payNowIdRes != null) ? payNowIdRes.Status : "I do not know status because of PayNowIdResponse is null."));
                Console.WriteLine("Message:" + resDto.MerrMsg);
                Console.WriteLine("Result Code:" + resDto.VResultCode);
                Console.WriteLine("PayNowId ProcId:" + ((payNowIdRes != null) ? payNowIdRes.ProcessId : "I do not know processId because of PayNowIdResponse is null."));

            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception Occured !");
                Console.WriteLine("Exception Message " + ex.Message);
            }

        }
    }
}