﻿using System.Text;
using CoreMvcApplication.Models.BankViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class BankController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public BankController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<BankController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region BankList

    [HttpGet]
    public IActionResult BankList(BankListViewModel model)
    {
        var requestDto = new SearchRequestDto {MasterNames = new[] {"bankFinancialInstInfo"}};

        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
        var responseData =
            _mdkTransactionService.ExecuteMdkTransaction<SearchResponseDto>(tran, requestDto);

        // テーブルに金融機関情報レコードを追加
        model.Bankfinancials = responseData.MasterInfos?.MasterInfo?.First()?.Masters?.BankFinancialInstInfo;

        return View(model);
    }

    #endregion

    #region Thanks

    [HttpPost]
    public IActionResult Thanks()
    {
        return View();
    }

    #endregion

    #region BankAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new BankAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Name1 = model.Name1,
                Name2 = model.Name2,
                Kana1 = model.Kana1,
                Kana2 = model.Kana2,
                PayLimit = model.PayLimit,
                Contents = model.Contents,
                ContentsKana = model.ContentsKana
            };
            if ("netbank-pc".Equals(model.ServiceOptionType)) requestDto.TermUrl = _settings.Bank.TermUrl;
            if (!string.IsNullOrWhiteSpace(model.PushUrl)) requestDto.PushUrl = model.PushUrl;
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<BankAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult",
                new {orderId = model.OrderId, serviceOptionType = model.ServiceOptionType});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId, string serviceOptionType)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<BankAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.ServiceOptionType = serviceOptionType;
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ShunoKikanNo = responseData.ShunoKikanNo;
            model.CustomerNo = responseData.CustomerNo;
            model.ConfirmNo = responseData.ConfirmNo;
            model.Url = responseData.Url;
            if (SampleConstants.TxnSuccessCode.Equals(responseData.Mstatus) && !"atm".Equals(serviceOptionType))
            {
                var view = responseData.View;
                if (!string.IsNullOrEmpty(view))
                {
                    return Content(view, "text/html", Encoding.UTF8);
                }

                model.Skno = responseData.ShunoKikanNo;
                model.Bptn = responseData.BillPattern;
                model.Bill = responseData.Bill;
            }
        }

        return View(model);
    }

    #endregion
}