﻿using CoreMvcApplication.Models.CardViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class CardController : Controller
{
    #region CardAuthorize

    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;

    public CardController(IMdkTransactionService mdkTransactionService, ILogger<CardController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _logger = logger;
    }

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var jpo = "";
            if (new[] {"10", "80"}.Contains(model.Jpo1))
                jpo = model.Jpo1;
            else if ("61".Equals(model.Jpo1) || string.IsNullOrWhiteSpace(model.Jpo2))
                jpo = model.Jpo1 + "C" + model.Jpo2;

            var requestDto = new CardAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Token = model.Token,
                WithCapture = model.WithCapture
            };
            if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CardAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ReqCardNumber = responseData.ReqCardNumber;
            model.ResAuthCode = responseData.ResAuthCode;
        }

        return View(model);
    }

    #endregion


    #region CardCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new CardCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CardCaptureResponseDto>(tran, requestDto);

            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion


    #region CardCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new CardCancelRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount?.ToString()
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CardCancelResponseDto>(tran, requestDto);

            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion


    #region CardReAuthorize

    [HttpGet]
    public IActionResult ReAuthorize()
    {
        var model = new ReAuthorizeViewModel
        {
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult ReAuthorize(ReAuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var jpo = "";
            if (new[] {"10", "80"}.Contains(model.Jpo1))
                jpo = model.Jpo1;
            else if ("61".Equals(model.Jpo1) || string.IsNullOrWhiteSpace(model.Jpo2))
                jpo = model.Jpo1 + "C" + model.Jpo2;

            var requestDto = new CardReAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                OriginalOrderId = model.OriginalOrderId,
                WithCapture = model.WithCapture
            };
            if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;

            if (!string.IsNullOrWhiteSpace(model.SecurityCode)) requestDto.SecurityCode = model.SecurityCode;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CardReAuthorizeResponseDto>(tran, requestDto);

            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ReAuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult ReAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardReAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.ReqCardNumber = responseData.ReqCardNumber;
            model.ResAuthCode = responseData.ResAuthCode;
        }

        return View(model);
    }

    #endregion CardReAuthorize
}