﻿using CoreMvcApplication.Models.CvsViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class CvsController : Controller
{
    #region CvsAuthorize

    private readonly SampleSettings _settings;
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;

    public CvsController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<CvsController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    [HttpGet]
    public IActionResult Authorize(uint price = 10)
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            PushUrl = _settings.Cvs.PushUrl
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new CvsAuthorizeRequestDto
            {
                ServiceOptionType = model.ServiceOptionType,
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Name1 = model.Name1,
                Name2 = model.Name2,
                TelNo = model.TelNo,
                PayLimit = model.PayLimit,
                PayLimitHhmm = model.PayLimitHhmm,
                PaymentType = "0" //現在はリザーブパラメータのため無条件に "0" を設定してください。(2010.05.22)
            };
            if (!string.IsNullOrWhiteSpace(model.PushUrl)) requestDto.PushUrl = model.PushUrl;
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CvsAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CvsAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.HaraikomiUrl = responseData.HaraikomiUrl;
            model.ReceiptNo = responseData.ReceiptNo;
        }

        return View(model);
    }

    #endregion


    #region CvsCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new CvsCancelRequestDto
            {
                OrderId = model.OrderId,
                ServiceOptionType = model.ServiceOptionType
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CvsCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CvsCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion
}