﻿using System.Text;
using CoreMvcApplication.Models.MasterpassViewModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class MasterPassController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public MasterPassController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<MasterPassController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region MasterPassLogin

    [HttpGet]
    public IActionResult Login(uint quantity1 = 1, uint value1 = 10, string description1 = "テスト商品")
    {
        var model = new LoginViewModel
        {
            OrderId = Utils.GenerateOrderId(),
            Quantity1 = quantity1,
            Value1 = value1,
            Description1 = description1,
            SuccessUrl = _settings.MasterPass.SuccessUrl,
            CancelUrl = _settings.MasterPass.CancelUrl,
            ErrorUrl = _settings.MasterPass.ErrorUrl
        };
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Login(LoginViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new MasterpassLoginRequestDto
            {
                OrderId = model.OrderId,
                ItemAmount = model.ItemAmount.ToString(),
                ItemId = model.ItemId,
                Description1 = model.Description1,
                Quantity1 = model.Quantity1.ToString(),
                Value1 = model.Value1.ToString(),
                ImageUrl1 = model.ImageUrl1,
                Description2 = model.Description2,
                Quantity2 = model.Quantity2?.ToString(),
                Value2 = model.Value2?.ToString(),
                ImageUrl2 = model.ImageUrl2,
                Description3 = model.Description3,
                Quantity3 = model.Quantity3.ToString(),
                Value3 = model.Value3.ToString(),
                ImageUrl3 = model.ImageUrl3,
                Description4 = model.Description4,
                Quantity4 = model.Quantity4.ToString(),
                Value4 = model.Value4.ToString(),
                ImageUrl4 = model.ImageUrl4,
                Description5 = model.Description5,
                Quantity5 = model.Quantity5.ToString(),
                Value5 = model.Value5.ToString(),
                ImageUrl5 = model.ImageUrl5,
                Description6 = model.Description6,
                Quantity6 = model.Quantity6.ToString(),
                Value6 = model.Value6.ToString(),
                ImageUrl6 = model.ImageUrl6,
                Description7 = model.Description7,
                Quantity7 = model.Quantity7.ToString(),
                Value7 = model.Value7.ToString(),
                ImageUrl7 = model.ImageUrl7,
                Description8 = model.Description8,
                Quantity8 = model.Quantity8.ToString(),
                Value8 = model.Value8.ToString(),
                ImageUrl8 = model.ImageUrl8,
                Description9 = model.Description9,
                Quantity9 = model.Quantity9.ToString(),
                Value9 = model.Value9.ToString(),
                ImageUrl9 = model.ImageUrl9,
                Description10 = model.Description10,
                Quantity10 = model.Quantity10.ToString(),
                Value10 = model.Value10.ToString(),
                ImageUrl10 = model.ImageUrl10,
                SuppressShippingAddress = model.SuppressShippingAddress,
                ShippingLocationProfile = model.ShippingLocationProfile,
                SuccessUrl = model.SuccessUrl,
                CancelUrl = model.CancelUrl,
                ErrorUrl = model.ErrorUrl
            };

            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<MasterpassLoginResponseDto>(tran, requestDto);
                if (responseData.Mstatus == "success")
                    return Content(responseData.ResponseContents, "text/html", Encoding.UTF8);

                var resultViewModel = new ResultViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                };
                return View("Result", resultViewModel);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        return View(model);
    }

    [HttpPost]
    public IActionResult Result([FromForm] IFormCollection posted)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(posted, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new ResultViewModel
        {
            Mstatus = posted["mstatus"],
            OrderId = posted["orderId"],
            VResultCode = posted["vResultCode"]
        };
        return View(model);
    }

    #endregion

    #region MasterPassAuthorize

    [HttpPost]
    public IActionResult Authorize([FromForm] IFormCollection posted)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(posted, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new AuthorizeViewModel
        {
            OrderId = posted["orderId"],
            CardNumber = posted["cardNumber"],
            ShippingName = posted["shippingName"],
            ShippingPhone = posted["shippingPhone"],
            ShippingCity = posted["shippingCity"],
            ShippingCountry = posted["shippingCountry"],
            ShippingCountrySubdivision = posted["shippingCountrySubdivision"],
            ShippingAddress1 = posted["shippingAddress1"],
            ShippingAddress2 = posted["shippingAddress2"],
            ShippingAddress3 = posted["shippingAddress3"],
            ShippingPostalCode = posted["shippingPostalCode"]
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AuthorizeExec(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new MasterpassAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                WithCapture = model.WithCapture
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MasterpassAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View("Authorize", model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MasterpassAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.AuthorizeDatetime = responseData.AuthorizeDatetime;
            model.AuthCode = responseData.AuthCode;
        }

        return View("Result", model);
    }

    #endregion

    #region MasterPassCapture

    [HttpGet]
    public IActionResult Capture()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Capture(CaptureViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new MasterpassCaptureRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MasterpassCaptureResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CaptureResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CaptureResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MasterpassCaptureResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CaptureDatetime = responseData.CaptureDatetime;
        }

        return View(model);
    }

    #endregion

    #region MasterPassCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new MasterpassCancelRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString()
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<MasterpassCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MasterpassCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.CancelDatetime = responseData.CancelDatetime;
        }

        return View(model);
    }

    #endregion
}