﻿using System.Text;
using CoreMvcApplication.Models.OricoscViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;

namespace CoreMvcApplication.Controllers;

public class OricoscController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public OricoscController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<OricoscController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    [HttpGet]
    public IActionResult Authorize()
    {
        var orderId = Utils.GenerateOrderId();
        var model = new AuthorizeViewModel
        {
            OrderId = orderId,
            OricoOrderNo = orderId,
            ItemName1 = "テスト商品1",
            ItemCount1 = 1,
            ItemAmount1 = 500
        };
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (string.IsNullOrEmpty(model.ItemName2))
        {
            ModelState.Remove("ItemCount2");
            ModelState.Remove("ItemAmount2");
        }

        if (string.IsNullOrEmpty(model.ItemName3))
        {
            ModelState.Remove("ItemCount3");
            ModelState.Remove("ItemAmount3");
        }

        if (string.IsNullOrEmpty(model.ItemName4))
        {
            ModelState.Remove("ItemCount4");
            ModelState.Remove("ItemAmount4");
        }

        if (string.IsNullOrEmpty(model.ItemName5))
        {
            ModelState.Remove("ItemCount5");
            ModelState.Remove("ItemAmount5");
        }

        if (ModelState.IsValid)
        {
            var requestData = new OricoscAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                OricoOrderNo = model.OricoOrderNo,
                UserNo = model.UserNo,
                ItemName1 = model.ItemName1,
                ItemCount1 = model.ItemCount1?.ToString(),
                ItemAmount1 = model.ItemAmount1?.ToString(),
                ItemName2 = model.ItemName2,
                ItemCount2 = model.ItemCount2?.ToString(),
                ItemAmount2 = model.ItemAmount2?.ToString(),
                ItemName3 = model.ItemName3,
                ItemCount3 = model.ItemCount3?.ToString(),
                ItemAmount3 = model.ItemAmount3?.ToString(),
                ItemName4 = model.ItemName4,
                ItemCount4 = model.ItemCount4?.ToString(),
                ItemAmount4 = model.ItemAmount4?.ToString(),
                ItemName5 = model.ItemName5,
                ItemCount5 = model.ItemCount5?.ToString(),
                ItemAmount5 = model.ItemAmount5?.ToString(),
                TotalItemAmount = model.TotalItemAmount.ToString(),
                TotalCarriage = model.TotalCarriage?.ToString(),
                Amount = model.Amount.ToString(),
                Deposit = model.Deposit?.ToString(),
                ShippingZipCode = model.ShippingZipCode,
                HandlingContractNo = model.HandlingContractNo,
                ContractDocumentKbn = model.ContractDocumentKbn,
                WebDescriptionId = model.WebDescriptionId,
                MerchantRedirectionUrl = _settings.OricoSc.MerchantRedirectionUrl
            };


            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<OricoscAuthorizeResponseDto>(tran, requestData);

                if (responseData.Mstatus == "success")
                    return Content(responseData.ResResponseContents, "text/html", Encoding.UTF8);

                var resultViewModel = new ResultViewModel
                {
                    MerrMsg = responseData.MerrMsg,
                    Mstatus = responseData.Mstatus,
                    OrderId = responseData.OrderId,
                    VResultCode = responseData.VResultCode
                };
                return View("AuthorizeResult", resultViewModel);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult Result(
        [FromQuery(Name = "serviceType")] string serviceType,
        [FromQuery(Name = "mstatus")] string mstatus,
        [FromQuery(Name = "vResultCode")] string vResultCode,
        [FromQuery(Name = "merrMsg")] string merrMsg,
        [FromQuery(Name = "marchTxn")] string marchTxn,
        [FromQuery(Name = "orderId")] string orderId,
        [FromQuery(Name = "custTxn")] string custTxn,
        [FromQuery(Name = "receiptNo")] string receiptNo
    )
    {
        var model = new ResultViewModel
        {
            MerrMsg = merrMsg,
            Mstatus = mstatus,
            OrderId = orderId,
            ReceiptNo = receiptNo,
            VResultCode = vResultCode
        };
        return View(model);
    }
}