﻿using System.Text;
using CoreMvcApplication.Models.PayNowIdModels;
using CoreMvcApplication.Resources;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class PayNowIdController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public PayNowIdController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<PayNowIdController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region AccountEntry

    [HttpGet]
    public IActionResult AccountEntry()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AccountEntry(AccountEntryViewModel model)
    {
        if (ModelState.IsValid)
        {
            if (model.EntryType == "1")
            {
                var requestDto = new AccountAddRequestDto
                {
                    AccountId = model.AccountId
                };
                if (!string.IsNullOrEmpty(model.CreateDate)) requestDto.CreateDate = model.CreateDate;

                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<AccountAddResponseDto>(tran, requestDto);
                TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                TempData[model.AccountId + "_EntryType"] = model.EntryType;
                return RedirectToAction("AccountEntryResult", new {accountId = model.AccountId});
            }
            else
            {
                var requestDto = new AccountRestoreRequestDto
                {
                    AccountId = model.AccountId
                };
                if (!string.IsNullOrEmpty(model.CreateDate)) requestDto.CreateDate = model.CreateDate;

                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<AccountRestoreResponseDto>(tran, requestDto);
                TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                TempData[model.AccountId + "_EntryType"] = model.EntryType;
                return RedirectToAction("AccountEntryResult", new {accountId = model.AccountId});
            }
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult AccountEntryResult(string accountId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(accountId))
        {
            var entryType = TempData.Peek(accountId + "_EntryType")?.ToString();

            AbstractPayNowIdResponseDto responseData;

            if (entryType == "1")
                responseData =
                    JsonConvert.DeserializeObject<AccountAddResponseDto>(TempData.Peek(accountId).ToString());
            else
                responseData =
                    JsonConvert.DeserializeObject<AccountRestoreResponseDto>(TempData.Peek(accountId).ToString());

            model.PayNowIdProcId = responseData.PayNowIdResponse?.ProcessId;
            model.PayNowIdStatus = responseData.PayNowIdResponse?.Status;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region EntryAndCard

    [HttpGet]
    public IActionResult EntryAndCard(uint price = 10)
    {
        var model = new EntryAndCardViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult EntryAndCard(EntryAndCardViewModel model)
    {
        if (ModelState.IsValid)
        {
            var jpo = "";
            if (new[] {"10", "80"}.Contains(model.Jpo1))
                jpo = model.Jpo1;
            else if ("61".Equals(model.Jpo1) || string.IsNullOrWhiteSpace(model.Jpo2))
                jpo = model.Jpo1 + "C" + model.Jpo2;

            var requestDto = new CardAuthorizeRequestDto
            {
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                Token = model.Token,
                WithCapture = model.WithCapture
            };
            if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;

            if (!string.IsNullOrWhiteSpace(model.AccountId)) requestDto.AccountId = model.AccountId;

            if (!string.IsNullOrWhiteSpace(model.GroupId)) requestDto.GroupId = model.GroupId;

            if (!string.IsNullOrWhiteSpace(model.StartDate)) requestDto.StartDate = model.StartDate;

            if (!string.IsNullOrWhiteSpace(model.EndDate)) requestDto.EndDate = model.EndDate;

            if (model.OneTimeAmount != null) requestDto.OneTimeAmount = model.OneTimeAmount.ToString();

            if (model.RecurringAmount != null) requestDto.RecurringAmount = model.RecurringAmount.ToString();

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<CardAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("EntryAndCardResult", new {orderId = model.OrderId});
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult EntryAndCardResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.PayNowIdProcId = responseData.PayNowIdResponse?.ProcessId;
            model.PayNowIdStatus = responseData.PayNowIdResponse?.Status;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region AccountWithdraw

    [HttpGet]
    public IActionResult AccountWithdraw()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult AccountWithdraw(AccountWithdrawViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new AccountDeleteRequestDto
            {
                AccountId = model.AccountId
            };
            if (!string.IsNullOrEmpty(model.DeleteDate)) requestDto.DeleteDate = model.DeleteDate;

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<AccountDeleteResponseDto>(tran, requestDto);
            TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AccountWithdrawResult", new {accountId = model.AccountId});
        }

        return View(model);
    }


    [HttpGet]
    public IActionResult AccountWithdrawResult(string accountId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(accountId))
        {
            var responseData =
                JsonConvert.DeserializeObject<AccountDeleteResponseDto>(TempData.Peek(accountId).ToString());
            model.PayNowIdProcId = responseData.PayNowIdResponse?.ProcessId;
            model.PayNowIdStatus = responseData.PayNowIdResponse?.Status;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region CardManage

    [HttpGet]
    public IActionResult CardManage()
    {
        var model = new CardManageViewModel();
        return View(model);
    }

    private enum CardManageMode
    {
        CardInfoGet = 1,
        CardInfoDelete = 2,
        CardInfoAdd = 3,
        CardInfoUpdate = 4
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult CardManage(CardManageViewModel model)
    {
        CardManageMode mode;
        if (model.Mode == "1")
        {
            mode = CardManageMode.CardInfoGet;
            ModelState.Remove("Token");
            ModelState.Remove("CardId");
        }
        else if (model.Mode == "2")
        {
            mode = CardManageMode.CardInfoDelete;
            ModelState.Remove("Token");
        }
        else
        {
            if (string.IsNullOrWhiteSpace(model.CardId))
            {
                mode = CardManageMode.CardInfoAdd;
                ModelState.Remove("CardId");
            }
            else
            {
                mode = CardManageMode.CardInfoUpdate;
            }
        }

        TempData[model.AccountId + "_Mode"] = mode.ToString();

        if (ModelState.IsValid)
            switch (mode)
            {
                case CardManageMode.CardInfoGet:
                {
                    var requestDto = new CardInfoGetRequestDto
                    {
                        AccountId = model.AccountId
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoGetResponseDto>(tran, requestDto);
                    if (responseData.Mstatus != SampleConstants.TxnSuccessCode)
                    {
                        ModelState.AddModelError("CardInfoGetRequestError", responseData.MerrMsg);
                    }
                    else
                    {
                        var cardInfos = responseData.PayNowIdResponse?.Account?.CardInfo;
                        if (cardInfos != null)
                            foreach (var cardInfo in cardInfos)
                                model.CardIds.Add(new SelectListItem
                                {
                                    Value = cardInfo.CardId,
                                    Text = $"{cardInfo.CardNumber} : {cardInfo.CardExpire}"
                                });

                        return View(model);
                    }

                    break;
                }
                case CardManageMode.CardInfoDelete:
                {
                    var requestDto = new CardInfoDeleteRequestDto
                    {
                        AccountId = model.AccountId,
                        CardId = model.CardId
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoDeleteResponseDto>(tran, requestDto);
                    TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                    return RedirectToAction("CardManageResult", new {accountId = model.AccountId});
                }
                case CardManageMode.CardInfoAdd:
                {
                    var requestDto = new CardInfoAddRequestDto
                    {
                        AccountId = model.AccountId,
                        Token = model.Token
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoAddResponseDto>(tran, requestDto);
                    TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                    return RedirectToAction("CardManageResult", new {accountId = model.AccountId});
                }
                case CardManageMode.CardInfoUpdate:
                {
                    var requestDto = new CardInfoUpdateRequestDto
                    {
                        AccountId = model.AccountId,
                        CardId = model.CardId,
                        Token = model.Token
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoUpdateResponseDto>(tran, requestDto);
                    TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                    return RedirectToAction("CardManageResult", new {accountId = model.AccountId});
                }
            }

        return View(model);
    }


    [HttpGet]
    public IActionResult CardManageResult(string accountId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(accountId))
        {
            var type = TempData.Peek(accountId + "_Mode")?.ToString();

            AbstractPayNowIdResponseDto responseData = null;

            switch (type)
            {
                case "CardInfoAdd":
                    responseData =
                        JsonConvert.DeserializeObject<CardInfoAddResponseDto>(TempData.Peek(accountId).ToString());
                    break;
                case "CardInfoDelete":
                    responseData =
                        JsonConvert.DeserializeObject<CardInfoDeleteResponseDto>(
                            TempData.Peek(accountId).ToString());
                    break;
                case "CardInfoUpdate":
                    responseData =
                        JsonConvert.DeserializeObject<CardInfoUpdateResponseDto>(
                            TempData.Peek(accountId).ToString());
                    break;
            }

            model.PayNowIdProcId = responseData?.PayNowIdResponse?.ProcessId;
            model.PayNowIdStatus = responseData?.PayNowIdResponse?.Status;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region Recurring

    [HttpGet]
    public IActionResult Recurring()
    {
        var model = new RecurringViewModel();
        return View(model);
    }

    private enum RecurringMode
    {
        CardInfoGet = 1,
        NewCard = 2,
        SelectedCard = 3
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Recurring(RecurringViewModel model)
    {
        RecurringMode mode;
        if (model.Mode == "1")
        {
            mode = RecurringMode.CardInfoGet;
            ModelState.Remove("Token");
            ModelState.Remove("CardId");
            ModelState.Remove("GroupId");
        }
        else
        {
            if (string.IsNullOrWhiteSpace(model.CardId))
            {
                mode = RecurringMode.NewCard;
                ModelState.Remove("CardId");
            }
            else
            {
                mode = RecurringMode.SelectedCard;
                ModelState.Remove("Token");
            }
        }

        TempData[model.AccountId + "_ChargeMode"] = model.ChargeMode;

        if (ModelState.IsValid)
            switch (mode)
            {
                case RecurringMode.CardInfoGet:
                {
                    var requestDto = new CardInfoGetRequestDto
                    {
                        AccountId = model.AccountId
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoGetResponseDto>(tran, requestDto);
                    if (responseData.Mstatus != SampleConstants.TxnSuccessCode)
                    {
                        ModelState.AddModelError("CardInfoGetRequestError", responseData.MerrMsg);
                    }
                    else
                    {
                        var cardInfos = responseData.PayNowIdResponse?.Account?.CardInfo;
                        if (cardInfos != null)
                            foreach (var cardInfo in cardInfos)
                                model.CardIds.Add(new SelectListItem
                                {
                                    Value = cardInfo.CardId,
                                    Text = $"{cardInfo.CardNumber} : {cardInfo.CardExpire}"
                                });

                        return View(model);
                    }

                    break;
                }
                default:
                {
                    if (model.ChargeMode == "1")
                    {
                        var requestDto = new RecurringAddRequestDto
                        {
                            AccountId = model.AccountId,
                            GroupId = model.GroupId,
                            StartDate = model.StartDate,
                            EndDate = model.EndDate,
                            OneTimeAmount = model.OneTimeAmount?.ToString(),
                            Amount = model.Amount?.ToString()
                        };
                        if (!string.IsNullOrEmpty(model.CardId))
                            requestDto.CardId = model.CardId;
                        else
                            requestDto.Token = model.Token;

                        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                        var responseData =
                            _mdkTransactionService.ExecuteMdkTransaction<RecurringAddResponseDto>(tran, requestDto);
                        TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                        return RedirectToAction("RecurringResult", new {accountId = model.AccountId});
                    }
                    else
                    {
                        var requestDto = new RecurringUpdateRequestDto
                        {
                            AccountId = model.AccountId,
                            GroupId = model.GroupId,
                            EndDate = model.EndDate,
                            OneTimeAmount = model.OneTimeAmount?.ToString(),
                            Amount = model.Amount?.ToString()
                        };
                        if (!string.IsNullOrEmpty(model.CardId))
                            requestDto.CardId = model.CardId;
                        else
                            requestDto.Token = model.Token;

                        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                        var responseData =
                            _mdkTransactionService.ExecuteMdkTransaction<RecurringUpdateResponseDto>(tran, requestDto);
                        TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                        return RedirectToAction("RecurringResult", new {accountId = model.AccountId});
                    }
                }
            }

        return View(model);
    }

    [HttpGet]
    public IActionResult RecurringResult(string accountId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(accountId))
        {
            var type = TempData.Peek(accountId + "_ChargeMode")?.ToString();

            AbstractPayNowIdResponseDto responseData = null;

            switch (type)
            {
                case "1":
                    responseData =
                        JsonConvert.DeserializeObject<RecurringAddResponseDto>(TempData.Peek(accountId).ToString());
                    break;
                case "2":
                    responseData =
                        JsonConvert.DeserializeObject<RecurringUpdateResponseDto>(TempData.Peek(accountId)
                            .ToString());
                    break;
            }

            model.PayNowIdProcId = responseData?.PayNowIdResponse?.ProcessId;
            model.PayNowIdStatus = responseData?.PayNowIdResponse?.Status;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region RecurringTerminate

    [HttpGet]
    public IActionResult RecurringTerminate()
    {
        var model = new RecurringTerminateViewModel();
        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult RecurringTerminate(RecurringTerminateViewModel model)
    {
        if (model.Mode == "1") ModelState.Remove("GroupId");

        if (ModelState.IsValid)
        {
            if (model.Mode == "1")
            {
                var requestDto = new RecurringGetRequestDto
                {
                    AccountId = model.AccountId
                };
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<RecurringGetResponseDto>(tran, requestDto);
                if (responseData.Mstatus != SampleConstants.TxnSuccessCode)
                {
                    ModelState.AddModelError("CardInfoGetRequestError", responseData.MerrMsg);
                }
                else
                {
                    var recurringCharges = responseData.PayNowIdResponse?.Account?.RecurringCharge;
                    if (recurringCharges != null)
                    {
                        var cardInfos = responseData.PayNowIdResponse?.Account?.CardInfo;

                        foreach (var recurringCharge in recurringCharges)
                        {
                            var cardInfo = cardInfos?.FirstOrDefault(c => c.CardId == recurringCharge.CardId);
                            model.GroupIds.Add(new SelectListItem
                            {
                                Value = recurringCharge.GroupId,
                                Text =
                                    $"{recurringCharge.GroupId} : {cardInfo?.CardNumber} : {cardInfo?.CardExpire}"
                            });
                        }
                    }

                    return View(model);
                }
            }
            else
            {
                var requestDto = new RecurringDeleteRequestDto
                {
                    AccountId = model.AccountId,
                    GroupId = model.GroupId,
                    EndDate = model.EndDate,
                    FinalCharge = model.FinalCharge ? "1" : "0"
                };
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseData =
                    _mdkTransactionService.ExecuteMdkTransaction<RecurringDeleteResponseDto>(tran, requestDto);
                TempData[model.AccountId] = JsonConvert.SerializeObject(responseData);
                return RedirectToAction("RecurringTerminateResult", new {accountId = model.AccountId});
            }
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult RecurringTerminateResult(string accountId)
    {
        var model = new ResultViewModel();
        AbstractPayNowIdResponseDto responseData = null;
        if (TempData.ContainsKey(accountId))
            responseData =
                JsonConvert.DeserializeObject<RecurringDeleteResponseDto>(TempData.Peek(accountId).ToString());

        model.PayNowIdProcId = responseData?.PayNowIdResponse?.ProcessId;
        model.PayNowIdStatus = responseData?.PayNowIdResponse?.Status;
        model.VResultCode = responseData?.VResultCode;
        model.MerrMsg = responseData?.MerrMsg;

        return View(model);
    }

    #endregion

    #region CardAuthorize

    [HttpGet]
    public IActionResult CardAuthorize(uint price = 10)
    {
        var model = new CardAuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };
        return View(model);
    }

    private enum AuthorizeMode
    {
        CardInfoGet = 1,
        NewCard = 2,
        SelectedCard = 3
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult CardAuthorize(CardAuthorizeViewModel model)
    {
        AuthorizeMode mode;
        if (model.Mode == "1")
        {
            mode = AuthorizeMode.CardInfoGet;
            ModelState.Remove("OrderId");
            ModelState.Remove("Token");
            ModelState.Remove("Amount");
            ModelState.Remove("WithCapture");
            ModelState.Remove("Jpo1");
            ModelState.Remove("CardId");
        }
        else
        {
            if (string.IsNullOrWhiteSpace(model.CardId))
            {
                mode = AuthorizeMode.NewCard;
                ModelState.Remove("CardId");
            }
            else
            {
                mode = AuthorizeMode.SelectedCard;
                ModelState.Remove("Token");
            }
        }

        if (ModelState.IsValid)
            switch (mode)
            {
                case AuthorizeMode.CardInfoGet:
                {
                    var requestDto = new CardInfoGetRequestDto
                    {
                        AccountId = model.AccountId
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoGetResponseDto>(tran, requestDto);
                    if (responseData.Mstatus != SampleConstants.TxnSuccessCode)
                    {
                        ModelState.AddModelError("CardInfoGetRequestError", responseData.MerrMsg);
                    }
                    else
                    {
                        var cardInfos = responseData.PayNowIdResponse?.Account?.CardInfo;
                        if (cardInfos != null)
                            foreach (var cardInfo in cardInfos)
                                model.CardIds.Add(new SelectListItem
                                {
                                    Value = cardInfo.CardId,
                                    Text = $"{cardInfo.CardNumber} : {cardInfo.CardExpire}"
                                });

                        return View(model);
                    }

                    break;
                }
                default:
                {
                    var jpo = "";
                    if (new[] {"10", "80"}.Contains(model.Jpo1))
                        jpo = model.Jpo1;
                    else if ("61".Equals(model.Jpo1) || string.IsNullOrWhiteSpace(model.Jpo2))
                        jpo = model.Jpo1 + "C" + model.Jpo2;

                    var requestDto = new CardAuthorizeRequestDto
                    {
                        AccountId = model.AccountId,
                        OrderId = model.OrderId,
                        Amount = model.Amount.ToString(),
                        WithCapture = model.WithCapture
                    };
                    if (!string.IsNullOrEmpty(model.CardId))
                        requestDto.CardId = model.CardId;
                    else
                        requestDto.Token = model.Token;

                    if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;

                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardAuthorizeResponseDto>(tran, requestDto);
                    TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
                    return RedirectToAction("CardAuthorizeResult", new {orderId = model.OrderId});
                }
            }

        return View(model);
    }

    [HttpGet]
    public IActionResult CardAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<CardAuthorizeResponseDto>(TempData.Peek(orderId).ToString());

            model.OrderId = responseData?.OrderId;
            model.Mstatus = responseData?.Mstatus;
            model.PayNowIdStatus = responseData?.PayNowIdResponse?.Status;
            model.VResultCode = responseData?.VResultCode;
            model.MerrMsg = responseData?.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region MpiAuthorize

    [HttpGet]
    public IActionResult MpiAuthorize(uint price = 10)
    {
        var model = new MpiAuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId()
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult MpiAuthorize(MpiAuthorizeViewModel model,
        [FromHeader(Name = "Accept")] string accept, [FromHeader(Name = "User-Agent")] string userAgent)
    {
        AuthorizeMode mode;
        if (model.Mode == "1")
        {
            mode = AuthorizeMode.CardInfoGet;
            ModelState.Remove("OrderId");
            ModelState.Remove("Token");
            ModelState.Remove("Amount");
            ModelState.Remove("WithCapture");
            ModelState.Remove("Jpo1");
            ModelState.Remove("DeviceChannel");
            ModelState.Remove("CardholderName");
            ModelState.Remove("CardId");
        }
        else
        {
            if (string.IsNullOrWhiteSpace(model.CardId))
            {
                mode = AuthorizeMode.NewCard;
                ModelState.Remove("CardId");
            }
            else
            {
                mode = AuthorizeMode.SelectedCard;
                ModelState.Remove("Token");
            }
        }

        if (ModelState.IsValid)
            switch (mode)
            {
                case AuthorizeMode.CardInfoGet:
                {
                    var requestDto = new CardInfoGetRequestDto
                    {
                        AccountId = model.AccountId
                    };
                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<CardInfoGetResponseDto>(tran, requestDto);
                    if (responseData.Mstatus != SampleConstants.TxnSuccessCode)
                    {
                        ModelState.AddModelError("CardInfoGetRequestError", responseData.MerrMsg);
                    }
                    else
                    {
                        var cardInfos = responseData.PayNowIdResponse?.Account?.CardInfo;
                        if (cardInfos != null)
                            foreach (var cardInfo in cardInfos)
                                model.CardIds.Add(new SelectListItem
                                {
                                    Value = cardInfo.CardId,
                                    Text = $"{cardInfo.CardNumber} : {cardInfo.CardExpire}"
                                });

                        return View(model);
                    }

                    break;
                }
                default:
                {
                    var jpo = "";
                    if (new[] {"10", "80"}.Contains(model.Jpo1))
                        jpo = model.Jpo1;
                    else if ("61" == model.Jpo1 || string.IsNullOrWhiteSpace(model.Jpo2))
                        jpo = model.Jpo1 + "C" + model.Jpo2;

                    var requestDto = new MpiAuthorizeRequestDto
                    {
                        AccountId = model.AccountId,
                        OrderId = model.OrderId,
                        Amount = model.Amount.ToString(),
                        WithCapture = model.WithCapture,
                        ServiceOptionType = model.PaymentMode,
                        RedirectionUri = _settings.PayNowId.TermUrl,
                        HttpUserAgent = userAgent,
                        HttpAccept = accept,
                        DeviceChannel = model.DeviceChannel,
                        CardholderName = model.CardholderName
                    };
                    if (!string.IsNullOrEmpty(model.CardId))
                        requestDto.CardId = model.CardId;
                    else
                        requestDto.Token = model.Token;

                    if (!string.IsNullOrWhiteSpace(jpo)) requestDto.Jpo = jpo;

                    if (!string.IsNullOrWhiteSpace(model.PushUrl)) requestDto.PushUrl = model.PushUrl;

                    var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                    var responseData =
                        _mdkTransactionService.ExecuteMdkTransaction<MpiAuthorizeResponseDto>(tran, requestDto);
                    TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
                    TempData[model.OrderId + "_PaymentMode"] = model.PaymentMode;

                    return RedirectToAction("MpiAuthorizeResult", new {orderId = model.OrderId});
                }
            }

        return View(model);
    }

    [HttpGet]
    public IActionResult MpiAuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<MpiAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            if (responseData.Mstatus != SampleConstants.TxnFailureCode)
                return Redirect(responseData.AuthStartUrl);

            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    [HttpPost]
    public IActionResult Result([FromForm] IFormCollection posted)
    {
        var merchantConfig = _mdkTransactionService.GetMerchantConfigInstance(_logger);
        var merchantCcId = merchantConfig.GetValue(MerchantConfig.MerchantCcId);
        var merchantPw = merchantConfig.GetValue(MerchantConfig.MerchantSecretKey);
        if (string.IsNullOrWhiteSpace(merchantCcId) || string.IsNullOrWhiteSpace(merchantPw))
        {
            ViewData["Warning"] = Messages.MerchantCCIDorPasswordIsNull;
        }
        else
        {
            // パラメータ改ざんチェック
            if (!Utils.CheckAuthHash(posted, merchantCcId, merchantPw))
                ViewData["Warning"] = Messages.CheckAuthHashFalse;
        }

        var model = new MpiAuthorizeResultViewModel
        {
            OrderId = posted["OrderId"],
            RequestId = posted["RequestId"],
            MpiMStatus = posted["mpiMstatus"],
            CardMStatus = posted["cardMstatus"],
            VResultCode = posted["vResultCode"],
            ReqCurrencyUnit = posted["reqCurrencyUnit"],
            CardTransactionType = posted["cardTransactionType"],
            CenterRequestDate = posted["centerRequestDate"],
            CenterResponseDate = posted["centerResponseDate"],
            ConnectedCenterId = posted["connectedCenterId"],
            ReqCardNumber = posted["reqCardNumber"],
            ReqAmount = posted["reqAmount"],
            AcquirerCode = posted["acquirerCode"],
            AuthCode = posted["authCode"],
            DddMessageVersion = posted["dddMessageVersion"],
            DddTransactionId = posted["dddTransactionId"],
            DddTransactionStatus = posted["dddTransactionStatus"],
            DddCavvAlgorithm = posted["dddCavvAlgorithm"],
            DddCavv = posted["dddCavv"],
            DddEci = posted["dddEci"]
        };

        return View(model);
    }


    [HttpPost]
    public IActionResult Search([FromForm(Name = "orderId")] string orderId,
        [FromForm(Name = "requestId")] string requestId)
    {
        // 検索用リクエストパラメータを設定
        var requestDto = new SearchRequestDto
        {
            RequestId = requestId,
            NewerFlag = SampleConstants.MdkParamFalse
        };

        var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
        var responseData =
            _mdkTransactionService.ExecuteMdkTransaction<SearchResponseDto>(tran, requestDto);

        var model = new ResultViewModel
        {
            OrderId = orderId,
            Mstatus = responseData.Mstatus,
            VResultCode = responseData.VResultCode,
            MerrMsg = responseData.MerrMsg
        };

        // MPI認証結果とカード決済結果を取得します
        var transactionInfo = responseData.OrderInfos?.OrderInfo?.First()?.TransactionInfos?.TransactionInfo;

        var mpiStatus = "";
        var mpiVResultCode = "";
        var mpiTxnId = "";
        var cardStatus = "";
        var cardVResultCode = "";
        var cardTxnId = "";
        var paramPaymentMode = TempData.Peek(orderId + "_PaymentMode")?.ToString();
        if (transactionInfo != null)
            foreach (var info in transactionInfo)
            {
                // 検索結果用項目
                var mStatus = info.Mstatus;
                var vResultCode = info.VResultCode;
                var txnId = info.TxnId;
                var properInfo = info.ProperTransactionInfo;
                // 取引種別の取得
                var txnKind = "";
                if (properInfo != null) txnKind = properInfo.TxnKind;

                // MPI～カード決済までが実行された場合
                // このタイミングでリクエストIDによる検索を実行すると
                // txnKindが"mpi"と"card"のレコードがそれぞれ１件取得できる

                // MPI認証結果を取得(複数件の場合最新をチェックする)
                if (!string.IsNullOrWhiteSpace(txnId) && txnKind == "mpi")
                    if (string.IsNullOrWhiteSpace(mpiTxnId) || Convert.ToInt64(txnId) > Convert.ToInt64(mpiTxnId))
                    {
                        mpiStatus = mStatus;
                        mpiVResultCode = vResultCode;
                        mpiTxnId = txnId;
                    }

                // カード決済結果を取得（複数件の場合最新をチェックする）
                if (!string.IsNullOrWhiteSpace(txnId) && txnKind == "card")
                    if (string.IsNullOrWhiteSpace(cardTxnId) || Convert.ToInt64(txnId) > Convert.ToInt64(cardTxnId))
                    {
                        cardStatus = mStatus;
                        cardVResultCode = vResultCode;
                        cardTxnId = txnId;
                    }
            }

        // 基本は本人認証の結果
        model.Mstatus = mpiStatus;
        model.VResultCode = mpiVResultCode;

        // 本人認証に成功した場合、mStatusMPI の値は "success" になります
        if (mpiStatus == SampleConstants.TxnSuccessCode)
            // 本人認証成功で、決済種別＝「3D-Secure認証のみ(決済しない)」以外の
            // 場合、カード決済まで行われているため、完了画面に表示するのはカード決済の
            // 結果コードを表示する、というケース
            if (paramPaymentMode != "mpi-none")
            {
                model.Mstatus = cardStatus;
                model.VResultCode = cardVResultCode;
            }

        return View(model);
    }

    #endregion
}