﻿using System.Net;
using CoreMvcApplication.Models.PaymentMethodSelectViewModels;
using jp.veritrans.tercerog.mdk;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;

namespace CoreMvcApplication.Controllers;

public class PaymentMethodSelectController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly SampleSettings _settings;

    public PaymentMethodSelectController(IOptions<SampleSettings> settings,
        ILogger<PaymentMethodSelectController> logger)
    {
        _settings = settings.Value;
        _logger = logger;
    }

    public IActionResult Index()
    {
        var model = new MdkSettingsViewModel();

        // 接続先
        Uri uri;
        // 接続モード
        string dummyRequest;

        if (!string.IsNullOrWhiteSpace(_settings.Mdk.MdkIniPath))
        {
            var merchantConfig = new MerchantConfig(_logger, new FileInfo(_settings.Mdk.MdkIniPath));
            uri = new Uri(merchantConfig.GetValue(MerchantConfig.ServerUrl));
            dummyRequest = merchantConfig.GetValue(MerchantConfig.DummyRequest);
        }
        else
        {
            uri = new Uri(_settings.Mdk.Host);
            dummyRequest = _settings.Mdk.Dummy ? "1" : "0";
        }

        if (!IPAddress.TryParse(uri.Host, out _))
        {
            var task = Dns.GetHostEntryAsync(uri.Host);
            task.Wait();
            model.HostUrl = uri.Host + "/" + task.Result.AddressList.FirstOrDefault();
        }
        else
        {
            model.HostUrl = uri.Host;
        }

        model.DummyRequest = "1".Equals(dummyRequest) ? "ダミーリクエスト" : "本番リクエスト";
        return View(model);
    }

    [HttpPost]
    public IActionResult Select(string method)
    {
        // リダイレクトする
        switch (method)
        {
            case "card":
                return RedirectToAction("Authorize", "Card", new {price = 10});
            case "mpi":
                return RedirectToAction("Authorize", "Mpi", new {price = 10});
            case "cvs":
                return RedirectToAction("Authorize", "Cvs", new {price = 10});
            case "em":
                return RedirectToAction("Menu", "Em", new {price = 10});
            case "bank":
                return RedirectToAction("Authorize", "Bank", new {price = 10});
            case "paypal":
                return RedirectToAction("Authorize", "PayPal", new {price = 10});
            case "saison":
                return RedirectToAction("Authorize", "Saison", new {price = 10});
            case "upop":
                return RedirectToAction("Authorize", "Upop", new {price = 10});
            case "alipay":
                return RedirectToAction("Authorize", "Alipay", new {price = 10});
            case "carrier":
                return RedirectToAction("Menu", "Carrier", new {price = 10});
            case "oricosc":
                return RedirectToAction("Authorize", "Oricosc");
            case "rakuten":
                return RedirectToAction("Authorize", "Rakuten", new {price = 100, itemName = "テスト商品"});
            case "recruit":
                return RedirectToAction("Authorize", "Recruit", new {price = 100, itemName = "テスト商品"});
            case "linepay":
                return RedirectToAction("Authorize", "Linepay", new {price = 10, itemName = "テスト商品"});
            case "masterpass":
                return RedirectToAction("Login", "MasterPass",
                    new {quantity1 = 1, value1 = 10, description1 = "テスト商品"});
            case "virtualacc":
                return RedirectToAction("Authorize", "VirtualAcc", new
                    {price = 10, entryTransferNumber = "００００１", entryTransferName = "テストタロウ"});
            case "paypay":
                return RedirectToAction("Menu", "PayPay");
            case "amazonpay":
                return RedirectToAction("Authorize", "AmazonPay");
            default:
                return RedirectToAction("Index");
        }
    }
}