﻿using System.Text;
using jp.veritrans.tercerog.mdk;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;

namespace CoreMvcApplication.Controllers;

public class PushController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly SampleSettings _settings;

    public PushController(IOptions<SampleSettings> settings, ILogger<PushController> logger)
    {
        _settings = settings.Value;
        _logger = logger;
    }

    private ActionResult CheckPushData(PushType pushType, string contentHmac, IFormCollection data, Stream body)
    {
        try
        {
            if (string.IsNullOrEmpty(contentHmac)) return BadRequest();
            if (body.CanSeek) body.Position = 0;
            using (var reader = new StreamReader(body))
            {
                MerchantConfig merchantConfig;
                if (!string.IsNullOrWhiteSpace(_settings.Mdk.MdkIniPath))
                    merchantConfig = new MerchantConfig(_logger, new FileInfo(_settings.Mdk.MdkIniPath));
                else
                    merchantConfig = new MerchantConfig(_logger,
                        _settings.Mdk.MerchantCcId, _settings.Mdk.MerchantSecret,
                        _settings.Mdk.Dummy, hostUrl: _settings.Mdk.Host);
                if (MerchantUtility.CheckMessage(merchantConfig, reader.ReadToEnd(), contentHmac))
                {
                    OutputTrans(pushType, data);
                    return Ok();
                }
            }

            return BadRequest();
        }
        catch (Exception)
        {
            return StatusCode(500);
        }
    }

    [HttpPost]
    public ActionResult MpiPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Mpi, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult CvsPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Cvs, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult PayNowIdCardCleaningPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.PayNowIdCardCleaning, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult PayNowIdRecurringPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.PayNowIdRecurring, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult BankPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Bank, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult CarrierPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Carrier, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult EmPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Em, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult PayPalPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.PayPal, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult OricoscPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Oricosc, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult LinepayPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Linepay, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult RakutenPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Rakuten, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult RecruitPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Recruit, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult AlipayPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Alipay, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult UpopPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.Upop, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult PayPayPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.PayPay, contentHmac, data, Request.Body);
    }

    [HttpPost]
    public ActionResult AmazonPayPushReceipt(
        [FromHeader(Name = "content-hmac")] string contentHmac,
        [FromForm] IFormCollection data
    )
    {
        return CheckPushData(PushType.AmazonPay, contentHmac, data, Request.Body);
    }

    private void OutputTrans(PushType pushType, IFormCollection data)
    {
        var fileWriteString = new StringBuilder();
        fileWriteString.Append("---------------------------------------------------\n");
        fileWriteString.Append("pushTime:" + data["pushTime"] + "\n");
        fileWriteString.Append("pushId:" + data["pushId"] + "\n");
        fileWriteString.Append("numberOfNotify:" + data["numberOfNotify"] + "\n");
        fileWriteString.Append("---------------------------------------------------\n");

        var fileName = "PushReceipt.csv";

        var tranCount = Convert.ToInt32(data["numberOfNotify"]);

        for (var i = 0; i < tranCount; i++)
        {
            var count = string.Format("{0:d4}", i);
            string[] keys;

            switch (pushType)
            {
                case PushType.Mpi:
                    keys = new[]
                    {
                        "orderId", "txnType", "vResultCode", "mpiMstatus", "cardMstatus", "dummy"
                    };
                    fileName = "MpiPushReceipt.csv";
                    break;
                case PushType.Cvs:
                    keys = new[]
                    {
                        "orderId", "cvsType", "receiptNo", "receiptDate", "rcvAmount", "dummy"
                    };
                    fileName = "CvsPushReceipt.csv";
                    break;
                case PushType.PayNowIdCardCleaning:
                    keys = new[]
                    {
                        "accountId", "txnType", "receiptDate", "cardId", "cardNo", "cardExpire", "cardCheckStatus",
                        "dummy"
                    };
                    fileName = "PayNowIdCardCleaningPushReceipt.csv";
                    break;
                case PushType.PayNowIdRecurring:
                    keys = new[]
                    {
                        "accountId", "orderId", "txnType", "mstatus", "txnTime", "chargeGroupId", "chargeAmount",
                        "chargeDate", "dummy"
                    };
                    fileName = "PayNowIdRecurringePushReceipt.csv";
                    break;
                case PushType.Bank:
                    keys = new[]
                    {
                        "orderId", "kikanNo", "kigyoNo", "rcvDate", "customerNo", "confNo", "rcvAmount", "dummy"
                    };
                    fileName = "BankPushReceipt.csv";
                    break;
                case PushType.Carrier:
                    keys = new[]
                    {
                        "orderId", "txnType", "txnTime", "vResultCode", "mstatus", "crOrderId", "openId", "month",
                        "dummy"
                    };
                    fileName = "CarrierPushReceipt.csv";
                    break;
                case PushType.Em:
                    keys = new[]
                    {
                        "orderId", "emType", "receiptDate", "dummy"
                    };
                    fileName = "EmPushReceipt.csv";
                    break;
                case PushType.PayPal:
                    keys = new[]
                    {
                        "orderId", "mstatus", "txnType", "receivedDatetime", "amount", "payerId", "centerTxnId", "dummy"
                    };
                    fileName = "PayPalPushReceipt.csv";
                    break;
                case PushType.Oricosc:
                    keys = new[]
                    {
                        "orderId", "oricoOrderNo", "orderStateCode", "receiptNo", "approvalNo", "requestDate",
                        "loanPrincipal", "paymentCount", "dummy"
                    };
                    fileName = "OricoscPushReceipt.csv";
                    break;
                case PushType.Linepay:
                    keys = new[]
                    {
                        "orderId", "linepayOrderId", "txnType", "txnTime", "vResultCode", "mstatus", "dummy"
                    };
                    fileName = "LinepayPushReceipt.csv";
                    break;
                case PushType.Rakuten:
                    keys = new[]
                    {
                        "orderId", "rakutenOrderId", "txnType", "txnTime", "vResultCode", "mstatus", "balance",
                        "usedPoint", "rakutenApiErrorCode", "rakutenOrderErrorCode", "dummy", "rakutenApiErrorType",
                        "gatewayOrderId", "cardBrand", "cardLast4", "cardInstallments", "cardCvc", "card3ds",
                        "cancelExpirationTime", "captureExpirationTime", "updateExpirationTime",
                        "extendAuthExpirationTime", "customerId", "rakutenConsentOrderId"
                    };
                    fileName = "RakutenPushReceipt.csv";
                    break;
                case PushType.Recruit:
                    keys = new[]
                    {
                        "orderId", "recruitOrderId", "txnType", "txnTime", "vResultCode", "mstatus", "usePoint",
                        "givePoint", "recruitCoupon", "merchantCoupon", "dummy"
                    };
                    fileName = "RecruitPushReceipt.csv";
                    break;
                case PushType.Alipay:
                    keys = new[]
                    {
                        "orderId", "txnType", "vResultCode", "mstatus", "centerTradeId", "dummy"
                    };
                    fileName = "AlipayPushReceipt.csv";
                    break;
                case PushType.Upop:
                    keys = new[]
                    {
                        "orderId", "txnType", "mstatus", "traceNumber", "traceTime", "settleAmount", "settleDate",
                        "settleRate", "dummy", "vResultCode", "upopOrderId"
                    };
                    fileName = "UpopPushReceipt.csv";
                    break;
                case PushType.PayPay:
                    keys = new[]
                    {
                        "orderId", "txnType", "txnTime", "vResultCode", "mstatus", "paypayOrderId", "dummy"
                    };
                    fileName = "PayPayPushReceipt.csv";
                    break;
                case PushType.AmazonPay:
                    keys = new[]
                    {
                        "orderId", "txnType", "txnTime", "vResultCode", "mstatus", "centerOrderId",
                        "centerTransactionId", "refundableAmount", "refundableAmount", "balance", "dummy",
                        "accountingType", "originalOrderId", "consentAuthType"
                    };
                    fileName = "AmazonPayPushReceipt.csv";
                    break;
                default:
                    keys = new[] {"txnType", "dummy"};
                    fileName = "PushReceipt.csv";
                    break;
            }

            foreach (var key in keys) fileWriteString.Append(data[key + count] + ",");

            fileWriteString.Append("\n");
        }

        System.IO.File.AppendAllText(
            Path.Combine(_settings.Push.OutputDirectory, fileName), fileWriteString.ToString(), Encoding.UTF8);
    }

    private enum PushType
    {
        Mpi,
        Cvs,
        PayNowIdCardCleaning,
        PayNowIdRecurring,
        Bank,
        Carrier,
        Em,
        PayPal,
        Oricosc,
        Linepay,
        Rakuten,
        Recruit,
        Alipay,
        Upop,
        PayPay,
        AmazonPay,
    }
}