﻿using CoreMvcApplication.Models.SearchViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.Extensions.Options;

namespace CoreMvcApplication.Controllers;

public class SearchController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public SearchController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<SearchController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }


    // GET
    public ActionResult Index()
    {
        var model = new SearchViewModel
        {
            AvailableServiceType = GetServiceType(),
            AvailableCommands = GetCommands(),
            AvailableMStatus = GetMStatus(),
            AvailableOrderStatus = GetOrderStatus(),
            AvailableMpiDetailOrderType = GetMpiDetailOrderType(),
            AvailableMpiDeviceChannel = GetMpiDeviceChannel(),
            AvailableMpiAccountType = GetMpiAccountType(),
            AvailableEmType = GetEmType(),
            AvailableCvsType = GetCvsType(),
            AvailableUpopDetailOrderType = GetUpopDetailOrderType(),
            AvailableAlipayDetailOrderType = GetAlipayDetailOrderType(),
            AvailableAlipayPayType = GetAlipayPayType(),
            AvailableCarrierDetailOrderType = GetCarrierDetailOrderType(),
            AvailableCarrierCrServiceType = GetCarrierCrServiceType(),
            AvailableCarrierAccountingType = GetCarrierAccountingType(),
            AvailableCarrierItemType = GetCarrierItemType(),
            AvailableCarrierMpStatus = GetCarrierMpStatus(),
            AvailableCarrierTerminalKind = GetCarrierTerminalKind(),
            AvailableCarrierDetailCommandType = GetCarrierDetailCommandType(),
            AvailableRakutenDetailOrderType = GetRakutenDetailOrderType(),
            AvailableRakutenDetailCommandType = GetRakutenDetailCommandType(),
            AvailableRakutenPayType = GetRakutenPayType(),
            AvailableRakutenAccountingType = GetRakutenAccountingType(),
            AvailableRakutenConsentStatus = GetRakutenConsentStatus(),
            AvailableRecruitDetailOrderType = GetRecruitDetailOrderType(),
            AvailableRecruitDetailCommandType = GetRecruitDetailCommandType(),
            AvailableLinepayDetailOrderType = GetLinepayDetailOrderType(),
            AvailableLinepayDetailCommandType = GetLinepayDetailCommandType(),
            AvailableMasterpassDetailOrderType = GetMasterpassDetailOrderType(),
            AvailableMasterpassDetailCommandType = GetMasterpassDetailCommandType(),
            AvailableVirtualAccDetailOrderType = GetVirtualAccDetailOrderType(),
            AvailableVirtualAccDetailCommandType = GetVirtualAccDetailCommandType(),
            AvailablePayPayServiceType = GetPayPayServiceType(),
            AvailablePayPayDetailOrderType = GetPayPayDetailOrderType(),
            AvailablePayPayDetailCommandType = GetPayPayDetailCommandType(),
            AvailableAmazonPayDetailOrderType = GetAmazonPayDetailOrderType(),
            AvailableAmazonPayDetailCommandType = GetAmazonPayDetailCommandType(),
            AvailableAmazonPayConfirmScreenType = GetAmazonPayConfirmScreenType(),
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Index(SearchViewModel model)
    {
        if (ModelState.IsValid)
        {
            var searchParameter = new SearchParameters
            {
                Common = new CommonSearchParameter
                {
                    OrderId = model.OrderId,
                    Command = model.SelectedCommands.Any() ? model.SelectedCommands.ToArray() : null,
                    Mstatus = model.SelectedMStatus.Any() ? model.SelectedMStatus.ToArray() : null,
                    OrderStatus = model.SelectedOrderStatus.Any() ? model.SelectedOrderStatus.ToArray() : null,
                    TxnDatetime = new SearchRange
                    {
                        From = model.TxnDatetimeFrom,
                        To = model.TxnDatetimeTo
                    },
                    Amount = new SearchRange
                    {
                        From = model.AmountFrom?.ToString(),
                        To = model.AmountTo?.ToString()
                    }
                }
            };

            if (model.SelectedServiceType.Contains("card"))
                searchParameter.Card = new CardSearchParameter
                {
                    DddTransactionId = model.CardDddTransactionId
                };

            if (model.SelectedServiceType.Contains("mpi"))
                searchParameter.Mpi = new MpiSearchParameter
                {
                    Res3dTransactionId = model.MpiRes3DTransactionId,
                    Res3dTransactionStatus = model.MpiRes3DTransactionStatus,
                    Res3dEci = model.MpiRes3DEci,
                    DetailOrderType = model.SelectedMpiDetailOrderType.Any()
                        ? model.SelectedMpiDetailOrderType.ToArray()
                        : null,
                    DeviceChannel = model.SelectedMpiDeviceChannel.Any()
                        ? model.SelectedMpiDeviceChannel.ToArray()
                        : null,
                    AccountType = model.SelectedMpiAccountType.Any() ? model.SelectedMpiAccountType.ToArray() : null,
                };

            if (model.SelectedServiceType.Contains("em"))
                searchParameter.Em = new EmSearchParameter
                {
                    EmType = model.SelectedEmType.Any() ? model.SelectedEmType.ToArray() : null,
                    SettlementLimit = new SearchRange
                    {
                        From = model.EmSettlementLimitFrom,
                        To = model.EmSettlementLimitTo
                    },
                    CompleteDatetime = new SearchRange
                    {
                        From = model.EmCompleteDatetimeFrom,
                        To = model.EmCompleteDatetimeTo
                    },
                    ReceiptNo = model.EmReceiptNo,
                };

            if (model.SelectedServiceType.Contains("cvs"))
                searchParameter.Cvs = new CvsSearchParameter
                {
                    CvsType = model.SelectedCvsType.Any() ? model.SelectedCvsType.ToArray() : null,
                    PayLimit = new SearchRange
                    {
                        From = model.CvsPayLimitFrom,
                        To = model.CvsPayLimitTo
                    },
                    PayLimitDatetime = new SearchRange
                    {
                        From = model.CvsPayLimitDatetimeFrom,
                        To = model.CvsPayLimitDatetimeTo
                    },
                    PaidDatetime = new SearchRange
                    {
                        From = model.CvsPaidDatetimeFrom,
                        To = model.CvsPaidDatetimeTo
                    }
                };

            if (model.SelectedServiceType.Contains("bank"))
                searchParameter.Bank = new BankSearchParameter
                {
                    PayLimit = new SearchRange
                    {
                        From = model.BankPayLimitFrom,
                        To = model.BankPayLimitTo
                    },
                    ReceivedDatetime = new SearchRange
                    {
                        From = model.BankReceivedDatetimeFrom,
                        To = model.BankReceivedDatetimeTo
                    },
                    ShunoKikanNo = model.BankShunoKikanNo,
                    CustomerNo = model.BankCustomerNo
                };

            if (model.SelectedServiceType.Contains("paypal"))
                searchParameter.Paypal = new PaypalSearchParameter
                {
                    PaymentDatetime = new SearchRange
                    {
                        From = model.PaypalPaymentDatetimeFrom,
                        To = model.PaypalPaymentDatetimeTo
                    },
                    InvoiceId = model.PaypalInvoiceId,
                    PayerId = model.PaypalPayerId
                };

            if (model.SelectedServiceType.Contains("saison"))
                searchParameter.Saison = new SaisonSearchParameter
                {
                    TotalAmount = new SearchRange
                    {
                        From = model.SaisonTotalAmountFrom?.ToString(),
                        To = model.SaisonTotalAmountTo?.ToString()
                    },
                    WalletAmount = new SearchRange
                    {
                        From =
                            model.SaisonWalletAmountFrom?.ToString(),
                        To = model.SaisonWalletAmountTo?.ToString()
                    },
                    CardAmount = new SearchRange
                    {
                        From = model.SaisonCardAmountFrom?.ToString(),
                        To = model.SaisonCardAmountTo?.ToString()
                    }
                };

            if (model.SelectedServiceType.Contains("upop"))
                searchParameter.Upop = new UpopSearchParameter
                {
                    SettleDatetimeJp = new SearchRange
                    {
                        From = model.UpopSettleDatetimeJpFrom,
                        To = model.UpopSettleDatetimeJpTo
                    },
                    SettleDatetimeCn = new SearchRange
                    {
                        From = model.UpopSettleDatetimeCnFrom,
                        To = model.UpopSettleDatetimeCnTo
                    },
                    DetailOrderType = model.SelectedUpopDetailOrderType.Any()
                        ? model.SelectedUpopDetailOrderType.ToArray()
                        : null
                };

            if (model.SelectedServiceType.Contains("alipay"))
                searchParameter.Alipay = new AlipaySearchParameter
                {
                    CenterTradeId = model.AlipayCenterTradeId,
                    PaymentTime = new SearchRange
                    {
                        From = model.AlipayPaymentTimeFrom,
                        To = model.AlipayPaymentTimeTo
                    },
                    SettlementTime = new SearchRange
                    {
                        From = model.AlipaySettlementTimeFrom,
                        To = model.AlipaySettlementTimeTo
                    },
                    DetailOrderType = model.SelectedAlipayDetailOrderType.Any()
                        ? model.SelectedAlipayDetailOrderType.ToArray()
                        : null,
                    PayType = model.SelectedAlipayPayType.Any() ? model.SelectedAlipayPayType.ToArray() : null
                };

            if (model.SelectedServiceType.Contains("carrier"))
                searchParameter.Carrier = new CarrierSearchParameter
                {
                    DetailOrderType = model.SelectedCarrierDetailOrderType.Any()
                        ? model.SelectedCarrierDetailOrderType.ToArray()
                        : null,
                    CrServiceType = model.SelectedCarrierCrServiceType.Any()
                        ? model.SelectedCarrierCrServiceType.ToArray()
                        : null,
                    AccountingType = string.IsNullOrEmpty(model.SelectedCarrierAccountingType)
                        ? null
                        : model.SelectedCarrierAccountingType,
                    ItemType = model.SelectedCarrierItemType.Any() ? model.SelectedCarrierItemType.ToArray() : null,
                    MpStatus = model.SelectedCarrierMpStatus.Any() ? model.SelectedCarrierMpStatus.ToArray() : null,
                    TerminalKind = model.SelectedCarrierTerminalKind.Any()
                        ? model.SelectedCarrierTerminalKind.ToArray()
                        : null,
                    DetailCommandType = model.SelectedCarrierDetailCommandType.Any()
                        ? model.SelectedCarrierDetailCommandType.ToArray()
                        : null,
                    MpFirstDate = model.CarrierMpFirstDate,
                    MpDay = model.CarrierMpDay,
                    ItemId = model.CarrierItemId
                };

            if (model.SelectedServiceType.Contains("oricosc"))
                searchParameter.Oricosc = new OricoscSearchParameter
                {
                    OricoOrderNo = model.OricoscOricoOrderNo,
                    Amount = new SearchRange
                    {
                        From = model.OricoscAmountFrom?.ToString(),
                        To = model.OricoscAmountTo?.ToString()
                    },
                    TotalItemAmount = new SearchRange
                    {
                        From = model.OricoscTotalItemAmountFrom?.ToString(),
                        To = model.OricoscTotalItemAmountTo?.ToString()
                    }
                };

            if (model.SelectedServiceType.Contains("rakuten"))
                searchParameter.Rakuten = new RakutenSearchParameter
                {
                    DetailOrderType = model.SelectedRakutenDetailOrderType.Any()
                        ? model.SelectedRakutenDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedRakutenDetailCommandType.Any()
                        ? model.SelectedRakutenDetailCommandType.ToArray()
                        : null,
                    ItemId = model.RakutenItemId,
                    PayType = model.SelectedRakutenPayType.Any()
                        ? model.SelectedRakutenPayType.ToArray()
                        : null,
                    AccountingType = string.IsNullOrEmpty(model.SelectedRakutenAccountingType)
                        ? null
                        : model.SelectedRakutenAccountingType,
                    ConsentStatus = model.SelectedRakutenConsentStatus.Any()
                        ? model.SelectedRakutenConsentStatus.ToArray()
                        : null,
                    OriginalOrderId = model.RakutenOriginalOrderId
                };

            if (model.SelectedServiceType.Contains("recruit"))
                searchParameter.Recruit = new RecruitSearchParameter
                {
                    DetailOrderType = model.SelectedRecruitDetailOrderType.Any()
                        ? model.SelectedRecruitDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedRecruitDetailCommandType.Any()
                        ? model.SelectedRecruitDetailCommandType.ToArray()
                        : null,
                    ItemId = model.RecruitItemId
                };

            if (model.SelectedServiceType.Contains("linepay"))
                searchParameter.Linepay = new LinepaySearchParameter
                {
                    DetailOrderType = model.SelectedLinepayDetailOrderType.Any()
                        ? model.SelectedLinepayDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedLinepayDetailCommandType.Any()
                        ? model.SelectedLinepayDetailCommandType.ToArray()
                        : null,
                    ItemId = model.LinepayItemId
                };

            if (model.SelectedServiceType.Contains("masterpass"))
                searchParameter.Masterpass = new MasterpassSearchParameter
                {
                    DetailOrderType = model.SelectedMasterpassDetailOrderType.Any()
                        ? model.SelectedMasterpassDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedMasterpassDetailCommandType.Any()
                        ? model.SelectedMasterpassDetailCommandType.ToArray()
                        : null,
                    ItemId = model.MasterpassItemId,
                    AcquirerCode = model.MasterpassAcquirerCode
                };

            if (model.SelectedServiceType.Contains("virtualacc"))
                searchParameter.Virtualacc = new VirtualaccSearchParameter
                {
                    DetailOrderType = model.SelectedVirtualAccDetailOrderType.Any()
                        ? model.SelectedVirtualAccDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedVirtualAccDetailCommandType.Any()
                        ? model.SelectedVirtualAccDetailCommandType.ToArray()
                        : null,
                    EntryTransferName = model.VirtualAccEntryTransferName,
                    EntryTransferNumber = model.VirtualAccEntryTransferNumber,
                    AccountNumber = model.VirtualAccAccountNumber
                };

            if (model.SelectedServiceType.Contains("paypay"))
                searchParameter.Paypay = new PaypaySearchParameter
                {
                    DetailOrderType = model.SelectedPayPayDetailOrderType.Any()
                        ? model.SelectedPayPayDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedPayPayDetailCommandType.Any()
                        ? model.SelectedPayPayDetailCommandType.ToArray()
                        : null,
                    PaypayServiceType = model.SelectedPayPayServiceType.Any()
                        ? model.SelectedPayPayServiceType.ToArray()
                        : null,
                    AccountingType = string.IsNullOrEmpty(model.SelectedPayPayAccountingType)
                        ? null
                        : model.SelectedPayPayAccountingType
                };

            if (model.SelectedServiceType.Contains("amazonpay"))
                searchParameter.Amazonpay = new AmazonpaySearchParameter
                {
                    DetailOrderType = model.SelectedAmazonPayDetailOrderType.Any()
                        ? model.SelectedAmazonPayDetailOrderType.ToArray()
                        : null,
                    DetailCommandType = model.SelectedAmazonPayDetailCommandType.Any()
                        ? model.SelectedAmazonPayDetailCommandType.ToArray()
                        : null,
                    CenterOrderId = model.AmazonPayCenterOrderId,
                    CenterTransactionId = model.AmazonPayCenterTransactionId,
                    PayConfirmScreenType = string.IsNullOrEmpty(model.SelectedAmazonPayConfirmScreenType)
                        ? null
                        : model.SelectedAmazonPayConfirmScreenType
                };

            var requestDto = new SearchRequestDto
            {
                RequestId = model.RequestId,
                NewerFlag = model.IsNewerTxn ? "true" : "false",
                ContainDummyFlag = "1",
                ServiceTypeCd = model.SelectedServiceType.Any() ? model.SelectedServiceType.ToArray() : null,
                SearchParameters = searchParameter
            };

            try
            {
                var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
                var responseDto =
                    _mdkTransactionService.ExecuteMdkTransaction<SearchResponseDto>(tran, requestDto);

                var orderInfos = responseDto.OrderInfos;
                ViewBag.OrderInfos = orderInfos;

                if (!string.IsNullOrEmpty(responseDto.MerrMsg))
                    ModelState.AddModelError("MDK_MerrMsg", responseDto.MerrMsg);
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("MDK_Exception", ex.Message);
            }
        }

        model.AvailableCommands = GetCommands();
        model.AvailableMStatus = GetMStatus();
        model.AvailableOrderStatus = GetOrderStatus();
        model.AvailableServiceType = GetServiceType();
        model.AvailableMpiDetailOrderType = GetMpiDetailOrderType();
        model.AvailableMpiDeviceChannel = GetMpiDeviceChannel();
        model.AvailableMpiAccountType = GetMpiAccountType();
        model.AvailableEmType = GetEmType();
        model.AvailableCvsType = GetCvsType();
        model.AvailableUpopDetailOrderType = GetUpopDetailOrderType();
        model.AvailableAlipayDetailOrderType = GetAlipayDetailOrderType();
        model.AvailableAlipayPayType = GetAlipayPayType();
        model.AvailableCarrierDetailOrderType = GetCarrierDetailOrderType();
        model.AvailableCarrierCrServiceType = GetCarrierCrServiceType();
        model.AvailableCarrierAccountingType = GetCarrierAccountingType();
        model.AvailableCarrierItemType = GetCarrierItemType();
        model.AvailableCarrierMpStatus = GetCarrierMpStatus();
        model.AvailableCarrierTerminalKind = GetCarrierTerminalKind();
        model.AvailableCarrierDetailCommandType = GetCarrierDetailCommandType();
        model.AvailableRakutenDetailOrderType = GetRakutenDetailOrderType();
        model.AvailableRakutenDetailCommandType = GetRakutenDetailCommandType();
        model.AvailableRakutenPayType = GetRakutenPayType();
        model.AvailableRakutenAccountingType = GetRakutenAccountingType();
        model.AvailableRakutenConsentStatus = GetRakutenConsentStatus();
        model.AvailableRecruitDetailOrderType = GetRecruitDetailOrderType();
        model.AvailableRecruitDetailCommandType = GetRecruitDetailCommandType();
        model.AvailableLinepayDetailOrderType = GetLinepayDetailOrderType();
        model.AvailableLinepayDetailCommandType = GetLinepayDetailCommandType();
        model.AvailableMasterpassDetailOrderType = GetMasterpassDetailOrderType();
        model.AvailableMasterpassDetailCommandType = GetMasterpassDetailCommandType();
        model.AvailableVirtualAccDetailOrderType = GetVirtualAccDetailOrderType();
        model.AvailableVirtualAccDetailCommandType = GetVirtualAccDetailCommandType();
        model.AvailablePayPayServiceType = GetPayPayServiceType();
        model.AvailablePayPayDetailOrderType = GetPayPayDetailOrderType();
        model.AvailablePayPayDetailCommandType = GetPayPayDetailCommandType();
        model.AvailableAmazonPayDetailOrderType = GetAmazonPayDetailOrderType();
        model.AvailableAmazonPayDetailCommandType = GetAmazonPayDetailCommandType();
        model.AvailableAmazonPayConfirmScreenType = GetAmazonPayConfirmScreenType();
        return View(model);
    }


    private OrderInfo GetOrderInfo(string orderId, string serviceTypeCd)
    {
        var searchParameter = new SearchParameters
        {
            Common = new CommonSearchParameter
            {
                OrderId = orderId
            }
        };

        var requestDto = new SearchRequestDto
        {
            NewerFlag = "false",
            ContainDummyFlag = "1",
            ServiceTypeCd = new[] {serviceTypeCd},
            SearchParameters = searchParameter
        };

        OrderInfo orderInfo = null;

        try
        {
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseDto =
                _mdkTransactionService.ExecuteMdkTransaction<SearchResponseDto>(tran, requestDto);

            orderInfo = responseDto.OrderInfos?.OrderInfo?.First();

            if (!string.IsNullOrEmpty(responseDto.MerrMsg))
                ModelState.AddModelError("MDK_MerrMsg", responseDto.MerrMsg);
        }
        catch (Exception ex)
        {
            ModelState.AddModelError("MDK_Exception", ex.Message);
        }

        return orderInfo;
    }

    [HttpGet]
    public ActionResult Carrier(string orderId)
    {
        return View(GetOrderInfo(orderId, "carrier"));
    }

    [HttpGet]
    public ActionResult Rakuten(string orderId)
    {
        return View(GetOrderInfo(orderId, "rakuten"));
    }

    [HttpGet]
    public ActionResult Recruit(string orderId)
    {
        return View(GetOrderInfo(orderId, "recruit"));
    }

    [HttpGet]
    public ActionResult Linepay(string orderId)
    {
        return View(GetOrderInfo(orderId, "linepay"));
    }

    [HttpGet]
    public ActionResult Masterpass(string orderId)
    {
        return View(GetOrderInfo(orderId, "masterpass"));
    }

    [HttpGet]
    public ActionResult VirtualAcc(string orderId)
    {
        return View(GetOrderInfo(orderId, "virtualacc"));
    }

    [HttpGet]
    public ActionResult Alipay(string orderId)
    {
        return View(GetOrderInfo(orderId, "alipay"));
    }

    [HttpGet]
    public ActionResult Upop(string orderId)
    {
        return View(GetOrderInfo(orderId, "upop"));
    }

    [HttpGet]
    public ActionResult PayPay(string orderId)
    {
        return View(GetOrderInfo(orderId, "paypay"));
    }

    [HttpGet]
    public ActionResult AmazonPay(string orderId)
    {
        return View(GetOrderInfo(orderId, "amazonpay"));
    }

    private static IList<SelectListItem> GetServiceType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "カード", Value = "card"},
            new() {Text = "MPI", Value = "mpi"},
            new() {Text = "電子マネー", Value = "em"},
            new() {Text = "コンビニ", Value = "cvs"},
            new() {Text = "銀行", Value = "bank"},
            new() {Text = "PayPal", Value = "paypal"},
            new() {Text = "Saison", Value = "saison"},
            new() {Text = "銀聯ネット決済(UPOP)", Value = "upop"},
            new() {Text = "Alipay決済", Value = "alipay"},
            new() {Text = "キャリア", Value = "carrier"},
            new() {Text = "ショッピングクレジット", Value = "oricosc"},
            new() {Text = "楽天ID決済", Value = "rakuten"},
            new() {Text = "リクルート", Value = "recruit"},
            new() {Text = "LINE Pay", Value = "linepay"},
            new() {Text = "MasterPass", Value = "masterpass"},
            new() {Text = "銀行振込決済", Value = "virtualacc"},
            new() {Text = "PayPay", Value = "paypay"},
            new() {Text = "Amazon Pay", Value = "amazonpay"},
        };
    }

    private static IList<SelectListItem> GetCommands()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Authorize", Value = "Authorize"},
            new() {Text = "ReAuthorize", Value = "ReAuthorize"},
            new() {Text = "Capture", Value = "Capture"},
            new() {Text = "Cancel", Value = "Cancel"},
            new() {Text = "Refund", Value = "Refund"},
            new() {Text = "Retry", Value = "Retry"},
            new() {Text = "Verify", Value = "Verify"},
            new() {Text = "Terminate", Value = "Terminate"},
            new() {Text = "Remove", Value = "Remove"},
            new() {Text = "UpdateAuthorize", Value = "UpdateAuthorize"}
        };
    }

    private static IList<SelectListItem> GetMStatus()
    {
        return new List<SelectListItem>
        {
            new() {Text = "success", Value = "success"},
            new() {Text = "failure", Value = "failure"},
            new() {Text = "pending", Value = "pending"}
        };
    }

    private static IList<SelectListItem> GetOrderStatus()
    {
        return new List<SelectListItem>
        {
            new() {Text = "end", Value = "end"},
            new() {Text = "end_presentation", Value = "end_presentation"},
            new() {Text = "pending", Value = "pending"},
            new() {Text = "expired", Value = "expired"},
            new() {Text = "error", Value = "error"},
            new() {Text = "validation_error", Value = "validation_error"},
            new() {Text = "initial", Value = "initial"}
        };
    }

    private static IList<SelectListItem> GetMpiDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "認可", Value = "auth"},
            new() {Text = "認可(条件付)", Value = "authc"},
            new() {Text = "検証", Value = "vd"},
            new() {Text = "検証(条件付)", Value = "vdc"},
            new() {Text = "検証(スキップ)", Value = "vds"},
            new() {Text = "本人認証", Value = "chg"}
        };
    }
    
    private static IList<SelectListItem> GetMpiDeviceChannel()
    {
        return new List<SelectListItem>
        {
            new() {Text = " 02:ブラウザベース", Value = "02"}
        };
    }

    private static IList<SelectListItem> GetMpiAccountType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "01:該当なし", Value = "01"},
            new() {Text = "02:クレジット", Value = "02"},
            new() {Text = "03:デビット", Value = "03"}
        };
    }

    private static IList<SelectListItem> GetEmType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "edy", Value = "edy"},
            new() {Text = "suica", Value = "suica"},
            new() {Text = "waon", Value = "waon"},
            new() {Text = "nanaco", Value = "tcc"},
        };
    }

    private static IList<SelectListItem> GetCvsType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "ｾﾌﾞﾝｲﾚﾌﾞﾝ", Value = "sej"},
            new() {Text = "ﾛｰｿﾝ､ﾌｧﾐﾘｰﾏｰﾄ、&nbsp;etc...", Value = "econ"},
            new() {Text = "ｻｰｸﾙKｻﾝｸｽ、ﾃﾞｲﾘｰﾔﾏｻﾞｷ&nbsp;etc...", Value = "other"}
        };
    }

    private static IList<SelectListItem> GetUpopDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "与信請求", Value = "a0"},
            new() {Text = "与信請求成功", Value = "a"},
            new() {Text = "与信請求失敗", Value = "ax"},
            new() {Text = "与信請求の取消", Value = "av"},
            new() {Text = "売上", Value = "pa"},
            new() {Text = "売上の取消", Value = "pav"},
            new() {Text = "返金", Value = "par"},
            new() {Text = "返金(残金なし)", Value = "pard"},
            new() {Text = "与信売上請求", Value = "ac0"},
            new() {Text = "与信売上請求成功", Value = "ac"},
            new() {Text = "与信売上請求失敗", Value = "acx"},
            new() {Text = "与信売上請求の取消", Value = "acv"},
            new() {Text = "返金(与信売上の返金，残りの返金可能金額は1円以上の場合)", Value = "acr"},
            new() {Text = "返金(与信売上の返金，残りの返金可能金額は0円になった場合)", Value = "acrd"}
        };
    }

    private static IList<SelectListItem> GetAlipayDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "決済請求", Value = "payment_request"},
            new() {Text = "決済請求成功", Value = "payment"},
            new() {Text = "返金申込み請求", Value = "refund_request"},
            new() {Text = "返金成功", Value = "refund"}
        };
    }

    private static IList<SelectListItem> GetAlipayPayType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "オンライン", Value = "0"},
            new() {Text = "バーコード(店舗スキャン型)", Value = "1"},
            new() {Text = "バーコード(消費者スキャン型)", Value = "2"}
        };
    }

    private static IList<SelectListItem> GetCarrierDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "抹消", Value = "Deregistration"},
            new() {Text = "継続終了", Value = "Terminate"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
            new() {Text = "与信有効期限切れ", Value = "ExpiredAuth"}
        };
    }

    private static IList<SelectListItem> GetCarrierCrServiceType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "au", Value = "au"},
            new() {Text = "docomo", Value = "docomo"},
            new() {Text = "ソフトバンクまとめて支払い(B)", Value = "sb_ktai"},
            new() {Text = "ソフトバンクまとめて支払い(A)", Value = "sb_matomete"},
            new() {Text = "フレッツまとめて支払い", Value = "flets"}
        };
    }

    private static IList<SelectListItem> GetCarrierAccountingType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "都度", Value = "0"},
            new() {Text = "継続", Value = "1"},
            new() {Text = "随時", Value = "4"}
        };
    }

    private static IList<SelectListItem> GetCarrierItemType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "デジタルコンテンツ", Value = "0"},
            new() {Text = "物販", Value = "1"},
            new() {Text = "役務", Value = "2"}
        };
    }

    private static IList<SelectListItem> GetCarrierMpStatus()
    {
        return new List<SelectListItem>
        {
            new() {Text = "継続中", Value = "1"},
            new() {Text = "登録抹消", Value = "8"},
            new() {Text = "継続終了", Value = "9"}
        };
    }

    private static IList<SelectListItem> GetCarrierTerminalKind()
    {
        return new List<SelectListItem>
        {
            new() {Text = "PC", Value = "0"},
            new() {Text = "スマートフォン", Value = "1"},
            new() {Text = "フィーチャーフォン", Value = "2"}
        };
    }

    private static IList<SelectListItem> GetCarrierDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "PreOpenId", Value = "PreOpenId"},
            new() {Text = "PostOpenId", Value = "PostOpenId"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "MAuth", Value = "MAuth"},
            new() {Text = "Deregistration", Value = "Deregistration"},
            new() {Text = "UserTerminatePreOpenId", Value = "UserTerminatePreOpenId"},
            new() {Text = "UserTerminatePostOpenId", Value = "UserTerminatePostOpenId"},
            new() {Text = "UserTerminatePreAuth", Value = "UserTerminatePreAuth"},
            new() {Text = "UserTerminate", Value = "UserTerminate"},
            new() {Text = "TerminateReq", Value = "TerminateReq"},
            new() {Text = "Terminate", Value = "Terminate"},
            new() {Text = "PostAuthReq", Value = "PostAuthReq"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "RefundPostAuthReq", Value = "RefundPostAuthReq"},
            new() {Text = "RefundPostAuth", Value = "RefundPostAuth"},
            new() {Text = "VoidPostAuthReq", Value = "VoidPostAuthReq"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "VoidAuthReq", Value = "VoidAuthReq"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "EmuAuth", Value = "EmuAuth"},
            new() {Text = "EmuPostAuthReq", Value = "EmuPostAuthReq"},
            new() {Text = "EmuPostAuth", Value = "EmuPostAuth"},
            new() {Text = "RefundAuthCaptureReq", Value = "RefundAuthCaptureReq"},
            new() {Text = "RefundAuthCapture", Value = "RefundAuthCapture"},
            new() {Text = "VoidAuthCaptureReq", Value = "VoidAuthCaptureReq"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
            new() {Text = "ExpiredAuth", Value = "ExpiredAuth"},
            new() {Text = "Consent", Value = "Consent"},
            new() {Text = "ConsentDeregistration", Value = "ConsentDeregistration"},
            new() {Text = "ConsentTerminateReq", Value = "ConsentTerminateReq"},
            new() {Text = "ConsentTerminate", Value = "ConsentTerminate"}
        };
    }

    private static IList<SelectListItem> GetRakutenDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上要求", Value = "PostAuthReq"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "売上失敗", Value = "PostAuthFail"},
            new() {Text = "取消要求(売上)", Value = "VoidPostAuthReq"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "変更要求(売上)", Value = "ChangePostAuthReq"},
            new() {Text = "変更(売上)", Value = "ChangePostAuth"},
            new() {Text = "取消要求(与信)", Value = "VoidAuthReq"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "変更要求(与信)", Value = "ChangeAuthReq"},
            new() {Text = "変更(与信)", Value = "ChangeAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "与信売上失敗", Value = "AuthCaptureFail"},
            new() {Text = "取消要求(与信売上)", Value = "VoidAuthCaptureReq"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
            new() {Text = "変更要求(与信売上)", Value = "ChangeAuthCaptureReq"},
            new() {Text = "変更(与信売上)", Value = "ChangeAuthCapture"},
            new() {Text = "与信無効", Value = "ExpiredAuth"},
            new() {Text = "承諾", Value = "Consent"}
        };
    }

    private static IList<SelectListItem> GetRakutenDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "Accept", Value = "Accept"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "AuthFail", Value = "AuthFail"},
            new() {Text = "PostAuthReq", Value = "PostAuthReq"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "PostAuthFail", Value = "PostAuthFail"},
            new() {Text = "VoidPostAuthReq", Value = "VoidPostAuthReq"},
            new() {Text = "VoidAuthReq", Value = "VoidAuthReq"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "ChangeAuthReq", Value = "ChangeAuthReq"},
            new() {Text = "ChangeAuth", Value = "ChangeAuth"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "AuthCaptureFail", Value = "AuthCaptureFail"},
            new() {Text = "EmuAuth", Value = "EmuAuth"},
            new() {Text = "EmuPostAuthReq", Value = "EmuPostAuthReq"},
            new() {Text = "EmuPostAuth", Value = "EmuPostAuth"},
            new() {Text = "EmuPostAuthFail", Value = "EmuPostAuthFail"},
            new() {Text = "VoidAuthCaptureReq", Value = "VoidAuthCaptureReq"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
            new() {Text = "ChangeAuthCaptureReq", Value = "ChangeAuthCaptureReq"},
            new() {Text = "ChangeAuthCapture", Value = "ChangeAuthCapture"},
            new() {Text = "ExpiredAuth", Value = "ExpiredAuth"},
            new() {Text = "RefundPostAuth", Value = "RefundPostAuth"},
            new() {Text = "RefundAuthCapture", Value = "RefundAuthCapture"},
            new() {Text = "Consent", Value = "Consent"}
        };
    }

    private static IList<SelectListItem> GetRakutenPayType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "オンライン", Value = "0"},
            new() {Text = "オンライン(V2)", Value = "2"}
        };
    }

    private static IList<SelectListItem> GetRakutenAccountingType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "都度", Value = "0"},
            new() {Text = "随時", Value = "1"}
        };
    }

    private static IList<SelectListItem> GetRakutenConsentStatus()
    {
        return new List<SelectListItem>
        {
            new() {Text = "初期状態", Value = "Init"},
            new() {Text = "承諾", Value = "Consent"},
        };
    }

    private static IList<SelectListItem> GetRecruitDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "変更(売上)", Value = "ChangePostAuth"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "変更(与信)", Value = "ChangeAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
            new() {Text = "変更(与信売上)", Value = "ChangeAuthCapture"},
            new() {Text = "与信無効", Value = "ExpiredAuth"}
        };
    }

    private static IList<SelectListItem> GetRecruitDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "PostAuthReq", Value = "PostAuthReq"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "ChangePostAuth", Value = "ChangePostAuth"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "EmuAuth", Value = "EmuAuth"},
            new() {Text = "EmuPostAuthReq", Value = "EmuPostAuthReq"},
            new() {Text = "EmuPostAuth", Value = "EmuPostAuth"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
            new() {Text = "ChangeAuthCapture", Value = "ChangeAuthCapture"},
            new() {Text = "ExpiredAuth", Value = "ExpiredAuth"},
            new() {Text = "ExtendAuth", Value = "ExtendAuth"}
        };
    }

    private static IList<SelectListItem> GetLinepayDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
            new() {Text = "与信無効", Value = "ExpiredAuth"}
        };
    }

    private static IList<SelectListItem> GetLinepayDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "QuitAuth", Value = "QuitAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "AuthFail", Value = "AuthFail"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
            new() {Text = "AuthCaptureFail", Value = "AuthCaptureFail"},
            new() {Text = "ExpiredAuth", Value = "ExpiredAuth"}
        };
    }

    private static IList<SelectListItem> GetMasterpassDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"}
        };
    }

    private static IList<SelectListItem> GetMasterpassDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "Login", Value = "Login"},
            new() {Text = "LoginNotify", Value = "LoginNotify"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"}
        };
    }

    private static IList<SelectListItem> GetVirtualAccDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "決済申込", Value = "Auth"},
            new() {Text = "消込済", Value = "Reconcile"},
            new() {Text = "取消", Value = "VoidAuth"}
        };
    }

    private static IList<SelectListItem> GetVirtualAccDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "DepositEntry", Value = "DepositEntry"},
            new() {Text = "DepositReverse", Value = "DepositReverse"},
            new() {Text = "Reconcile", Value = "Reconcile"},
            new() {Text = "UndoReconcile", Value = "UndoReconcile"},
            new() {Text = "VoidAuth", Value = "VoidAuth"}
        };
    }

    private static IList<SelectListItem> GetPayPayDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "init", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消(売上)", Value = "VoidPostAuth"},
            new() {Text = "取消(与信)", Value = "VoidAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
        };
    }

    private static IList<SelectListItem> GetPayPayDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "QuitAuth", Value = "QuitAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "RefundPostAuth", Value = "RefundPostAuth"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "RefundAuthCapture", Value = "RefundAuthCapture"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
        };
    }

    private static IList<SelectListItem> GetPayPayServiceType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "online", Value = "online"}
        };
    }

    private static IList<SelectListItem> GetAmazonPayDetailOrderType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "初期状態", Value = "Init"},
            new() {Text = "与信", Value = "Auth"},
            new() {Text = "売上", Value = "PostAuth"},
            new() {Text = "取消", Value = "VoidPostAuth"},
            new() {Text = "与信取消", Value = "VoidAuth"},
            new() {Text = "与信売上", Value = "AuthCapture"},
            new() {Text = "取消(与信売上)", Value = "VoidAuthCapture"},
        };
    }

    private static IList<SelectListItem> GetAmazonPayDetailCommandType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "Init", Value = "Init"},
            new() {Text = "PreAuth", Value = "PreAuth"},
            new() {Text = "Auth", Value = "Auth"},
            new() {Text = "PostAuthReq", Value = "PostAuthReq"},
            new() {Text = "PostAuth", Value = "PostAuth"},
            new() {Text = "RefundPostAuthReq", Value = "RefundPostAuthReq"},
            new() {Text = "RefundPostAuth", Value = "RefundPostAuth"},
            new() {Text = "VoidPostAuthReq", Value = "VoidPostAuthReq"},
            new() {Text = "VoidPostAuth", Value = "VoidPostAuth"},
            new() {Text = "VoidAuth", Value = "VoidAuth"},
            new() {Text = "AuthCapture", Value = "AuthCapture"},
            new() {Text = "RefundAuthCaptureReq", Value = "RefundAuthCaptureReq"},
            new() {Text = "RefundAuthCapture", Value = "RefundAuthCapture"},
            new() {Text = "VoidAuthCaptureReq", Value = "VoidAuthCaptureReq"},
            new() {Text = "VoidAuthCapture", Value = "VoidAuthCapture"},
        };
    }

    private static IList<SelectListItem> GetAmazonPayConfirmScreenType()
    {
        return new List<SelectListItem>
        {
            new() {Text = "決済サーバ決済確認画面利用", Value = "0"},
            new() {Text = "加盟店決済確認画面利用", Value = "1"}
        };
    }

}