﻿using CoreMvcApplication.Models.VirtualAccViewModels;
using CoreMvcApplication.Services;
using jp.veritrans.tercerog.mdk.dto;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;

namespace CoreMvcApplication.Controllers;

public class VirtualAccController : Controller
{
    private readonly ILogger<Controller> _logger;
    private readonly IMdkTransactionService _mdkTransactionService;
    private readonly SampleSettings _settings;

    public VirtualAccController(IMdkTransactionService mdkTransactionService, IOptions<SampleSettings> settings,
        ILogger<VirtualAccController> logger)
    {
        _mdkTransactionService = mdkTransactionService;
        _settings = settings.Value;
        _logger = logger;
    }

    #region VirtualAccAuthorize

    [HttpGet]
    public IActionResult Authorize(uint price = 10, string entryTransferNumber = "００００１",
        string entryTransferName = "テストタロウ")
    {
        var model = new AuthorizeViewModel
        {
            Amount = price,
            OrderId = Utils.GenerateOrderId(),
            EntryTransferName = entryTransferName,
            EntryTransferNumber = entryTransferNumber
        };

        return View(model);
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Authorize(AuthorizeViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccAuthorizeRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                EntryTransferName = model.EntryTransferName,
                EntryTransferNumber = model.EntryTransferNumber,
                TransferExpiredDate = model.TransferExpiredDate,
                AccountNumber = model.AccountNumber,
                AccountManageType = model.AccountManageType,
                BranchCode = model.BranchCode
            };

            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccAuthorizeResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("AuthorizeResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult AuthorizeResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccAuthorizeResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.TransferName = responseData.TransferName;
            model.BankName = responseData.BankName;
            model.BankCode = responseData.BankCode;
            model.BranchName = responseData.BranchName;
            model.BranchCode = responseData.BranchCode;
            model.AccountType = responseData.AccountType;
            model.AccountNumber = responseData.AccountNumber;
            model.AccountName = responseData.AccountName;
        }

        return View(model);
    }

    #endregion

    #region VirtualAccCancel

    [HttpGet]
    public IActionResult Cancel()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Cancel(CancelViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccCancelRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccCancelResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("CancelResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult CancelResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccCancelResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region VirtualAccDepositEntry

    [HttpGet]
    public IActionResult DepositEntry()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult DepositEntry(DepositEntryViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccDepositEntryRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId,
                Amount = model.Amount.ToString(),
                TransferName = model.TransferName,
                DepositDate = model.DepositDate,
                WithReconcile = model.WithReconcile,
                RegistrationMethod = model.RegistrationMethod
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccDepositEntryResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("DepositEntryResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult DepositEntryResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccDepositEntryResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.DepositId = responseData.DepositId;
            model.TotalDepositAmount = responseData.TotalDepositAmount;
            model.OrderStatus = responseData.OrderStatus;
        }

        return View(model);
    }

    #endregion

    #region VirtualAccDepositReverse

    [HttpGet]
    public IActionResult DepositReverse()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult DepositReverse(DepositReverseViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccDepositReverseRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId,
                DepositId = model.DepositId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccDepositReverseResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("DepositReverseResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult DepositReverseResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccDepositReverseResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
            model.TotalDepositAmount = responseData.TotalDepositAmount;
        }

        return View(model);
    }

    #endregion

    #region VirtualAccReconcile

    [HttpGet]
    public IActionResult Reconcile()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult Reconcile(ReconcileViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccReconcileRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccReconcileResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("ReconcileResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult ReconcileResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccReconcileResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion

    #region VirtualAccUndoReconcile

    [HttpGet]
    public IActionResult UndoReconcile()
    {
        return View();
    }

    [HttpPost]
    [ValidateAntiForgeryToken]
    public IActionResult UndoReconcile(UndoReconcileViewModel model)
    {
        if (ModelState.IsValid)
        {
            var requestDto = new VirtualaccUndoReconcileRequestDto
            {
                ServiceOptionType = MdkServiceConstants.VACC_RESONA,
                OrderId = model.OrderId
            };
            var tran = _mdkTransactionService.GetMdkTransactionInstance(_logger);
            var responseData =
                _mdkTransactionService.ExecuteMdkTransaction<VirtualaccUndoReconcileResponseDto>(tran, requestDto);
            TempData[model.OrderId] = JsonConvert.SerializeObject(responseData);
            return RedirectToAction("UndoReconcileResult", new {orderId = model.OrderId});
        }

        return View(model);
    }

    [HttpGet]
    public IActionResult UndoReconcileResult(string orderId)
    {
        var model = new ResultViewModel();

        if (TempData.ContainsKey(orderId))
        {
            var responseData =
                JsonConvert.DeserializeObject<VirtualaccUndoReconcileResponseDto>(TempData.Peek(orderId).ToString());
            model.OrderId = responseData.OrderId;
            model.Mstatus = responseData.Mstatus;
            model.VResultCode = responseData.VResultCode;
            model.MerrMsg = responseData.MerrMsg;
        }

        return View(model);
    }

    #endregion
}