﻿using System.ComponentModel.DataAnnotations;

namespace CoreMvcApplication.Models.AlipayViewModels;

public class AuthorizeViewModel
{
    [Display(Name = "取引ID")]
    [Required]
    [StringLength(100)]
    public string OrderId { get; set; }

    [Required]
    [Display(Name = "金額")]
    [Range(1, 9999999)]
    public uint Amount { get; set; }

    [Required]
    [Display(Name = "通貨単位")]
    [RegularExpression(@"\A(JPY|CNY)\z")]
    public string Currency { get; set; } = "JPY";

    [Required]
    [Display(Name = "決済種別")]
    [RegularExpression(@"\A(0|1|2)\z")]
    public string PayType { get; set; } = "0";

    [Required]
    [Display(Name = "ユーザ識別コード")]
    [RegularExpression(@"\A[0-9a-zA-Z]{1,32}\z")]
    public string IdentityCode { get; set; }

    [Required]
    [Display(Name = "商品名")]
    [StringLength(100)]
    public string CommodityName { get; set; }

    [Required]
    [Display(Name = "商品詳細")]
    [StringLength(200)]
    public string CommodityDescription { get; set; }

    [Required]
    [Display(Name = "商品ID")]
    [StringLength(64)]
    public string CommodityId { get; set; }

    [Required]
    [Display(Name = "決済完了時URL")]
    [StringLength(256)]
    [Url]
    public string SuccessUrl { get; set; }

    [Required]
    [Display(Name = "決済エラー時URL")]
    [StringLength(256)]
    [Url]
    public string ErrorUrl { get; set; }

    [Display(Name = "レスポンスタイプ")]
    [RegularExpression(@"\A(|0|1)\z")]
    public string ResponseType { get; set; }
}