﻿using System.ComponentModel.DataAnnotations;

namespace CoreMvcApplication.Models.CarrierViewModels;

public class AuthorizeViewModel
{
    [Required]
    [Display(Name = "取引ID")]
    [StringLength(100)]
    public string OrderId { get; set; }

    [Required]
    [Display(Name = "キャリア選択")]
    [RegularExpression(@"\A(docomo|au|sb_ktai|sb_matomete|flets)\z")]
    public string ServiceOptionType { get; set; }

    [Required] [Display(Name = "決済金額")] public uint? Amount { get; set; }

    [Required]
    [Display(Name = "端末種別")]
    [RegularExpression(@"\A(0|1|2)\z")]
    public string TerminalKind { get; set; }

    [Display(Name = "商品タイプ")]
    [RegularExpression(@"\A(0|1|2)\z")]
    public string ItemType { get; set; }

    [Required]
    [Display(Name = "都度継続区分")]
    [RegularExpression(@"\A(0|1|4)\z")]
    public string AccountingType { get; set; }

    [Required]
    [Display(Name = "与信方法")]
    [RegularExpression(@"\A(false|true)\z")]
    public string WithCapture { get; set; }

    [Required]
    [Display(Name = "本人認証(3Dセキュア)")]
    [RegularExpression(@"\A(0|1|2)\z")]
    public string D3Flag { get; set; }

    [Required]
    [Display(Name = "初回課金年月日")]
    [RegularExpression(@"\A[0-9]{8}\z")]
    public string MpFirstDate { get; set; }

    [Required]
    [Display(Name = "継続課金日(01-28)")]
    [RegularExpression(@"\A[0-9]{1,2}\z")]
    public string MpDay { get; set; }

    [Display(Name = "商品番号")]
    [RegularExpression(@"\A[0-9a-zA-Z\-]{1,20}\z")]
    public string ItemId { get; set; }

    [Display(Name = "商品情報")]
    [StringLength(24)]
    public string ItemInfo { get; set; }

    [Display(Name = "決済完了時URL")]
    [StringLength(256)]
    [Url]
    public string SuccessUrl { get; set; }

    [Display(Name = "決済キャンセル時URL")]
    [StringLength(256)]
    [Url]
    public string CancelUrl { get; set; }

    [Display(Name = "決済エラー時URL")]
    [StringLength(256)]
    [Url]
    public string ErrorUrl { get; set; }

    [Display(Name = "プッシュURL")]
    [StringLength(256)]
    [Url]
    public string PushUrl { get; set; }

    [Display(Name = "Open ID")]
    [StringLength(256)]
    public string OpenId { get; set; }

    [Display(Name = "フレッツエリア")]
    [RegularExpression(@"\A(0|1)\z")]
    public string FletsArea { get; set; }

    [Display(Name = "auIDログインフラグ")]
    [RegularExpression(@"\A(false|true)\z")]
    public string LoginAuId { get; set; }

}