﻿using System.ComponentModel.DataAnnotations;

namespace CoreMvcApplication.Models.MpiViewModels;

public class ReAuthorizeViewModel
{
    [Display(Name = "取引ID")]
    [Required]
    [StringLength(100)]
    public string OrderId { get; set; }

    [Required]
    [Display(Name = "金額")]
    [Range(1, 99999999)]
    public uint Amount { get; set; }

    [Required]
    [Display(Name = "決済種別")]
    [RegularExpression(@"\A(mpi\-none|mpi\-complete|mpi\-company|mpi\-merchant)\z")]
    public string PaymentMode { get; set; }

    [Required]
    [Display(Name = "与信方法")]
    [RegularExpression(@"\A(true|false)\z")]
    public string WithCapture { get; set; }

    [Display(Name = "元取引ID")]
    [Required]
    [StringLength(100)]
    public string OriginalOrderId { get; set; }

    [Display(Name = "支払方法")]
    [RegularExpression(@"\A(10|21|61|80)\z")]
    [Required]
    public string Jpo1 { get; set; }

    [Display(Name = "支払回数")]
    [RegularExpression(@"\A[0-9]{2}\z")]
    public string Jpo2 { get; set; }

    [Display(Name = "セキュリティコード")]
    [RegularExpression(@"\A[0-9]{3,4}\z")]
    public string SecurityCode { get; set; }

    [Display(Name = "Push通知先URL")]
    [Url]
    [StringLength(256)]
    public string PushUrl { get; set; }

    [Display(Name = "本人認証有効期限")] public string verifyTimeout { get; set; }

    [Display(Name = "デバイスチャネル")] public string deviceChannel { get; set; }

    [Display(Name = "アカウントタイプ")] public string accountType { get; set; }

    [Display(Name = "カード保有者名")] public string cardholderName { get; set; }
    
    [Display(Name = "カード保有者メールアドレス")] public string cardholderEmail { get; set; }

    [Display(Name = "カード保有者自宅電話番号国コード")] public string cardholderHomePhoneCountry { get; set; }

    [Display(Name = "カード保有者自宅電話番号")] public string cardholderHomePhoneNumber { get; set; }

    [Display(Name = "カード保有者携帯電話番号国コード")] public string cardholderMobilePhoneCountry { get; set; }

    [Display(Name = "カード保有者携帯電話番号")] public string cardholderMobilePhoneNumber { get; set; }

    [Display(Name = "カード保有者勤務先電話番号国コード")] public string cardholderWorkPhoneCountry { get; set; }

    [Display(Name = "カード保有者勤務先電話番号")] public string cardholderWorkPhoneNumber { get; set; }

    [Display(Name = "請求先住所_市区町村")] public string billingAddressCity { get; set; }

    [StringLength(3)]
    [Display(Name = "請求先住所_国")] public string billingAddressCountry { get; set; }

    [Display(Name = "請求先住所1")] public string billingAddressLine1 { get; set; }

    [Display(Name = "請求先住所2")] public string billingAddressLine2 { get; set; }

    [Display(Name = "請求先住所3")] public string billingAddressLine3 { get; set; }

    [Display(Name = "請求先郵便番号")] public string billingPostalCode { get; set; }

    [StringLength(3)]
    [Display(Name = "請求先住所_都道府県")] public string billingAddressState { get; set; }

    [Display(Name = "配送先住所_市区町村")] public string shippingAddressCity { get; set; }

    [StringLength(3)]
    [Display(Name = "配送先住所_国")] public string shippingAddressCountry { get; set; }

    [Display(Name = "配送先住所1")] public string shippingAddressLine1 { get; set; }

    [Display(Name = "配送先住所2")] public string shippingAddressLine2 { get; set; }

    [Display(Name = "配送先住所3")] public string shippingAddressLine3 { get; set; }

    [Display(Name = "配送先郵便番号")] public string shippingPostalCode { get; set; }

    [StringLength(3)]
    [Display(Name = "配送先住所_都道府県")] public string shippingAddressState { get; set; }

    [Display(Name = "消費者IPアドレス")] public string customerIp { get; set; }

    [Display(Name = "チャレンジ認証フラグ")]
    [RegularExpression(@"\A(true|false)\z")]
    public string withChallenge { get; set; }

}