﻿using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Mvc.Rendering;

namespace CoreMvcApplication.Models.PayNowIdModels;

public class MpiAuthorizeViewModel
{
    [Display(Name = "会員ID")]
    [StringLength(100)]
    [Required]
    public string AccountId { get; set; }

    [Display(Name = "取引ID")]
    [Required]
    [StringLength(100)]
    public string OrderId { get; set; }

    [Required]
    [Display(Name = "金額")]
    [Range(1, 99999999)]
    public uint Amount { get; set; }

    [Required]
    [Display(Name = "決済種別")]
    [RegularExpression(@"\A(mpi\-none|mpi\-complete|mpi\-company|mpi\-merchant)\z")]
    public string PaymentMode { get; set; }

    [Required]
    [Display(Name = "与信方法")]
    [RegularExpression(@"\A(true|false)\z")]
    public string WithCapture { get; set; }

    [Display(Name = "トークン")]
    [RegularExpression(@"\A[0-9a-f\-]{36}\z")]
    public string Token { get; set; }

    [Display(Name = "支払方法")]
    [RegularExpression(@"\A(10|21|61|80)\z")]
    [Required]
    public string Jpo1 { get; set; }

    [Display(Name = "支払回数")]
    [RegularExpression(@"\A[0-9]{2}\z")]
    public string Jpo2 { get; set; }

    [Display(Name = "デバイスチャネル")]
    public string DeviceChannel { get; set; }

    [Display(Name = "カード保有者名")]
    public string CardholderName { get; set; }

    [Display(Name = "Push通知先URL")]
    [Url]
    [StringLength(256)]
    public string PushUrl { get; set; }

    [Display(Name = "カード選択")]
    [StringLength(100)]
    [Required]
    public virtual string CardId { get; set; }

    public string Mode { get; set; }

    public List<SelectListItem> CardIds { get; set; } = new()
    {
        new() {Value = string.Empty, Text = "新しいカードを入力する"}
    };
}