﻿using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Mvc.Rendering;

namespace CoreMvcApplication.Models.PayNowIdModels;

public class RecurringViewModel
{
    [Display(Name = "トークン")]
    [RegularExpression(@"\A[0-9a-f\-]{36}\z")]
    [Required]
    public virtual string Token { get; set; }

    [Display(Name = "会員ID")]
    [StringLength(100)]
    [Required]
    public string AccountId { get; set; }

    [Display(Name = "カード選択")]
    [StringLength(100)]
    [Required]
    public virtual string CardId { get; set; }

    public string Mode { get; set; }

    public List<SelectListItem> CardIds { get; set; } = new()
    {
        new() {Value = string.Empty, Text = "新しいカードを登録する"}
    };

    [Required]
    [Display(Name = "処理モード")]
    [RegularExpression(@"\A(1|2)\z")]
    public string ChargeMode { get; set; }

    [Display(Name = "課金グループID")]
    [RegularExpression(@"\A[0-9a-zA-Z\.\-_@]{1,100}\z")]
    [Required]
    public string GroupId { get; set; }

    [Display(Name = "課金開始日")]
    [RegularExpression(@"\A[0-9]{8}\z")]
    public string StartDate { get; set; }

    [Display(Name = "課金終了日")]
    [RegularExpression(@"\A[0-9]{8}\z")]
    public string EndDate { get; set; }

    [Display(Name = "都度/初回課金金額")]
    [Range(1, 99999999)]
    public uint? OneTimeAmount { get; set; }

    [Display(Name = "継続課金金額")]
    [Range(1, 99999999)]
    public uint? Amount { get; set; }
}