﻿using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Mvc.Rendering;

namespace CoreMvcApplication.Models.SearchViewModels;

public class SearchViewModel
{
    [ServiceTypeAttribute(ErrorMessage = "決済サービスタイプを選択してください")]
    public IList<string> SelectedServiceType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableServiceType { get; set; }

    [Display(Name = "取引ID")]
    [StringLength(100)]
    public string OrderId { get; set; }

    [Display(Name = "リクエストID")]
    [StringLength(128)]
    public string RequestId { get; set; }

    public IList<string> SelectedCommands { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableCommands { get; set; }

    public IList<string> SelectedMStatus { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableMStatus { get; set; }

    public IList<string> SelectedOrderStatus { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableOrderStatus { get; set; }

    [Display(Name = "取引日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string TxnDatetimeFrom { get; set; }

    [Display(Name = "取引日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string TxnDatetimeTo { get; set; }

    [Display(Name = "金額(From)")] public uint? AmountFrom { get; set; }

    [Display(Name = "金額(To)")] public uint? AmountTo { get; set; }

    [Display(Name = "最新トランザクションのみ")] public bool IsNewerTxn { get; set; } = true;

    // カード
    [Display(Name = "3DﾄﾗﾝｻﾞｸｼｮﾝID")]
    [RegularExpression(@"\A[a-zA-Z0-9\+\-=]{28}\z")]
    public string CardDddTransactionId { get; set; }

    // MPI
    
    public IList<string> SelectedMpiDetailOrderType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableMpiDetailOrderType { get; set; }
    
    [Display(Name = "3DﾄﾗﾝｻﾞｸｼｮﾝID")]
    [RegularExpression(@"\A[a-zA-Z0-9\+\-=]{28}\z")]
    public string MpiRes3DTransactionId { get; set; }

    [Display(Name = "3Dﾄﾗﾝｻﾞｸｼｮﾝｽﾃｰﾀｽ")]
    [RegularExpression(@"\A[a-zA-Z0-9]{1}\z")]
    public string MpiRes3DTransactionStatus { get; set; }

    [Display(Name = "ECI")]
    [RegularExpression(@"\A[0-9]{2}\z")]
    public string MpiRes3DEci { get; set; }

    public IList<string> SelectedMpiDeviceChannel { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableMpiDeviceChannel { get; set; }

    public IList<string> SelectedMpiAccountType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableMpiAccountType { get; set; }

    // 電子マネー
    public IList<string> SelectedEmType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableEmType { get; set; }

    [Display(Name = "支払/受取期限(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string EmSettlementLimitFrom { get; set; }

    [Display(Name = "支払/受取期限(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string EmSettlementLimitTo { get; set; }

    [Display(Name = "支払完了日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string EmCompleteDatetimeFrom { get; set; }

    [Display(Name = "支払完了日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string EmCompleteDatetimeTo { get; set; }

    [Display(Name = "受付番号")]
    [StringLength(32)]
    public string EmReceiptNo { get; set; }

    // コンビニ
    public IList<string> SelectedCvsType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableCvsType { get; set; }

    [Display(Name = "支払期限(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPayLimitFrom { get; set; }

    [Display(Name = "支払期限(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPayLimitTo { get; set; }

    [Display(Name = "支払期限日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPayLimitDatetimeFrom { get; set; }

    [Display(Name = "支払期限日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPayLimitDatetimeTo { get; set; }

    [Display(Name = "入金受付日(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPaidDatetimeFrom { get; set; }

    [Display(Name = "入金受付日(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string CvsPaidDatetimeTo { get; set; }

    // 銀行
    [Display(Name = "支払期限(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string BankPayLimitFrom { get; set; }

    [Display(Name = "支払期限(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string BankPayLimitTo { get; set; }

    [Display(Name = "収納日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string BankReceivedDatetimeFrom { get; set; }

    [Display(Name = "収納日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string BankReceivedDatetimeTo { get; set; }

    [Display(Name = "収納機関番号")]
    [RegularExpression(@"\A[a-zA-Z0-9]{1,8}\z")]
    public string BankShunoKikanNo { get; set; }

    [Display(Name = "お客様番号")]
    [RegularExpression(@"\A[a-zA-Z0-9]{1,20}\z")]
    public string BankCustomerNo { get; set; }

    // PayPal
    [Display(Name = "支払日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string PaypalPaymentDatetimeFrom { get; set; }

    [Display(Name = "支払日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string PaypalPaymentDatetimeTo { get; set; }

    [Display(Name = "請求番号")]
    [RegularExpression(@"\A[a-zA-Z0-9]{1,127}\z")]
    public string PaypalInvoiceId { get; set; }

    [Display(Name = "顧客番号")]
    [RegularExpression(@"\A[a-zA-Z0-9]{1,13}\z")]
    public string PaypalPayerId { get; set; }

    // Saison
    [Display(Name = "合計金額(From)")] public int? SaisonTotalAmountFrom { get; set; }

    [Display(Name = "合計金額(To)")] public int? SaisonTotalAmountTo { get; set; }

    [Display(Name = "ウォレット決済金額(From)")] public int? SaisonWalletAmountFrom { get; set; }

    [Display(Name = "ウォレット決済金額(To)")] public int? SaisonWalletAmountTo { get; set; }

    [Display(Name = "カード決済金額(From)")] public int? SaisonCardAmountFrom { get; set; }

    [Display(Name = "カード決済金額(To)")] public int? SaisonCardAmountTo { get; set; }

    // UPOP
    [Display(Name = "決済日時(日本)(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string UpopSettleDatetimeJpFrom { get; set; }

    [Display(Name = "決済日時(日本)(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string UpopSettleDatetimeJpTo { get; set; }

    [Display(Name = "決済日時(中国)(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string UpopSettleDatetimeCnFrom { get; set; }

    [Display(Name = "決済日時(中国)(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string UpopSettleDatetimeCnTo { get; set; }

    public IList<string> SelectedUpopDetailOrderType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableUpopDetailOrderType { get; set; }

    // Alipay
    public IList<string> SelectedAlipayDetailOrderType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableAlipayDetailOrderType { get; set; }

    public IList<string> SelectedAlipayPayType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableAlipayPayType { get; set; }

    [Display(Name = "決済センターとの取引ID")]
    [StringLength(64)]
    public string AlipayCenterTradeId { get; set; }

    [Display(Name = "支払い日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string AlipayPaymentTimeFrom { get; set; }

    [Display(Name = "支払い日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string AlipayPaymentTimeTo { get; set; }

    [Display(Name = "清算日時(From)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string AlipaySettlementTimeFrom { get; set; }

    [Display(Name = "清算日時(To)")]
    [RegularExpression(@"\A[0-9]{12}\z")]
    public string AlipaySettlementTimeTo { get; set; }

    // Carrier
    public IList<string> SelectedCarrierDetailOrderType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableCarrierDetailOrderType { get; set; }

    public IList<string> SelectedCarrierCrServiceType { get; set; } = new List<string>();
    public IList<SelectListItem> AvailableCarrierCrServiceType { get; set; }

    public string SelectedCarrierAccountingType { get; set; } = "";

    public IList<SelectListItem> AvailableCarrierAccountingType { get; set; }

    public IList<string> SelectedCarrierItemType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableCarrierItemType { get; set; }

    public IList<string> SelectedCarrierMpStatus { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableCarrierMpStatus { get; set; }

    public IList<string> SelectedCarrierTerminalKind { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableCarrierTerminalKind { get; set; }

    public IList<string> SelectedCarrierDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableCarrierDetailCommandType { get; set; }

    [Display(Name = "初回課金年月日")]
    [RegularExpression(@"\A[0-9]{8}\z")]
    public string CarrierMpFirstDate { get; set; }

    [Display(Name = "継続課金日")]
    [RegularExpression(@"\A[0-9]{2}\z")]
    public string CarrierMpDay { get; set; }

    [Display(Name = "商品番号")]
    [RegularExpression(@"\A[0-9a-zA-Z]{1,18}\z")]
    public string CarrierItemId { get; set; }

    // OricoSC
    [Display(Name = "注文番号")]
    [StringLength(20)]
    public string OricoscOricoOrderNo { get; set; }

    [Display(Name = "支払金額合計(From)")] public int? OricoscAmountFrom { get; set; }

    [Display(Name = "支払金額合計(To)")] public int? OricoscAmountTo { get; set; }

    [Display(Name = "商品価格合計(From)")] public int? OricoscTotalItemAmountFrom { get; set; }

    [Display(Name = "商品価格合計(To)")] public int? OricoscTotalItemAmountTo { get; set; }

    // Rakuten
    public IList<string> SelectedRakutenDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRakutenDetailOrderType { get; set; }

    public IList<string> SelectedRakutenDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRakutenDetailCommandType { get; set; }

    [Display(Name = "商品番号")]
    [StringLength(64)]
    public string RakutenItemId { get; set; }

    public IList<string> SelectedRakutenPayType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRakutenPayType { get; set; }

    public string SelectedRakutenAccountingType { get; set; } = "";

    public IList<SelectListItem> AvailableRakutenAccountingType { get; set; }

    public IList<string> SelectedRakutenConsentStatus { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRakutenConsentStatus { get; set; }

    [Display(Name = "元取引ID")]
    [StringLength(100)]
    public string RakutenOriginalOrderId { get; set; }

    // Recruit
    public IList<string> SelectedRecruitDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRecruitDetailOrderType { get; set; }

    public IList<string> SelectedRecruitDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableRecruitDetailCommandType { get; set; }

    [Display(Name = "商品番号")]
    [StringLength(64)]
    public string RecruitItemId { get; set; }

    // LINE Pay
    public IList<string> SelectedLinepayDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableLinepayDetailOrderType { get; set; }

    public IList<string> SelectedLinepayDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableLinepayDetailCommandType { get; set; }

    [Display(Name = "商品番号")]
    [StringLength(64)]
    public string LinepayItemId { get; set; }

    // MasterPass
    public IList<string> SelectedMasterpassDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableMasterpassDetailOrderType { get; set; }

    public IList<string> SelectedMasterpassDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableMasterpassDetailCommandType { get; set; }

    [Display(Name = "商品番号")]
    [StringLength(64)]
    public string MasterpassItemId { get; set; }

    [Display(Name = "仕向け先コード")] public string MasterpassAcquirerCode { get; set; }

    // VirtualAcc
    public IList<string> SelectedVirtualAccDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableVirtualAccDetailOrderType { get; set; }

    public IList<string> SelectedVirtualAccDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableVirtualAccDetailCommandType { get; set; }

    [Display(Name = "登録時振込人名")] public string VirtualAccEntryTransferName { get; set; }

    [Display(Name = "登録時振込番号")]
    [StringLength(5)]
    public string VirtualAccEntryTransferNumber { get; set; }

    [Display(Name = "口座番号")]
    [RegularExpression(@"\A[0-9]{7}\z")]
    public string VirtualAccAccountNumber { get; set; }

    // PayPay
    public IList<string> SelectedPayPayDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailablePayPayDetailOrderType { get; set; }

    public IList<string> SelectedPayPayDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailablePayPayDetailCommandType { get; set; }

    public IList<string> SelectedPayPayServiceType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailablePayPayServiceType { get; set; }

    public string SelectedPayPayAccountingType { get; set; } = "";

    // AmazonPay
    public IList<string> SelectedAmazonPayDetailOrderType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableAmazonPayDetailOrderType { get; set; }

    public IList<string> SelectedAmazonPayDetailCommandType { get; set; } = new List<string>();

    public IList<SelectListItem> AvailableAmazonPayDetailCommandType { get; set; }

    [Display(Name = "決済センターの管理ID")]
    [StringLength(64)]
    public string AmazonPayCenterOrderId { get; set; }

    [Display(Name = "決済センターのトランザクション管理ID")]
    [StringLength(64)]
    public string AmazonPayCenterTransactionId { get; set; }
    
    public string SelectedAmazonPayConfirmScreenType { get; set; } = "";
    
    public IList<SelectListItem> AvailableAmazonPayConfirmScreenType { get; set; }
    
    
}