using CoreMvcApplication;
using CoreMvcApplication.Services;
using Serilog;
using Serilog.Events;

var builder = WebApplication.CreateBuilder(args);

Log.Logger = new LoggerConfiguration()
    .MinimumLevel.Information()
    .MinimumLevel.Override("Microsoft", LogEventLevel.Warning)
    .MinimumLevel.Override("System", LogEventLevel.Error)
    .Enrich.FromLogContext()
    .Enrich.WithThreadId()
    .Enrich.WithThreadName()
    .WriteTo.File("./mdk_.log",
        rollingInterval: RollingInterval.Day,
        outputTemplate:
        "[{Timestamp} {Level:u3}] <{ThreadId}><{ThreadName}> {Message:lj} {NewLine}{Exception}")
    .CreateLogger();

builder.Logging
    .AddSerilog(Log.Logger);

var configurationBuilder = builder.Configuration
    .AddJsonFile("appsettings.json")
    .Build();

builder.Services.Configure<SampleSettings>(configurationBuilder.GetSection("SampleSettings"));

// Add services to the container.
builder.Services.AddControllersWithViews()
    .AddSessionStateTempDataProvider()
    .AddRazorRuntimeCompilation();
builder.Services.AddDistributedMemoryCache();
builder.Services.AddSession(
    options =>
    {
        options.Cookie.SecurePolicy = CookieSecurePolicy.Always;
        options.Cookie.IsEssential = true;
        options.Cookie.SameSite = SameSiteMode.None;
    }
);
builder.Services.AddHttpContextAccessor();
builder.Services.AddTransient<IMdkTransactionService, MdkTransactionService>();

var app = builder.Build();

app.Use((context, func) =>
{
    context.Request.EnableBuffering();
    return func(context);
});

app.UseSession();

// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Home/Error");
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();
}
else
{
    app.UseDeveloperExceptionPage();
}

app.UseHttpsRedirection();
app.UseStaticFiles();
app.UseRouting();
app.UseAuthorization();

app.MapControllerRoute(
    "default",
    "{controller=Home}/{action=Index}/{id?}");

app.Run();