﻿#nullable enable
using System.Collections.Specialized;
using jp.veritrans.tercerog.mdk;
using Microsoft.Extensions.Primitives;

namespace CoreMvcApplication;

public class Utils
{
    private static readonly DateTime Jan1St1970 = new(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

    public static long CurrentTimeMillis()
    {
        return (long) (DateTime.UtcNow - Jan1St1970).TotalMilliseconds;
    }

    public static string GenerateOrderId()
    {
        return "dummy" + CurrentTimeMillis();
    }


    public static string FormatDateTime(string str)
    {
        var dateFormat08 = "{0}-{1}-{2}"; // yyyy-mm-ddフォーマット
        var dateFormat14 = "{0}-{1}-{2} {3}:{4}:{5}"; // yyyy-mm-dd hh:mi:ssフォーマット
        var dateFormat17 = "{0}-{1}-{2} {3}:{4}:{5}.{6}"; // yyyy-mm-dd hh:mi:ss.sssフォーマット
        var result = "";

        var sy = ""; // 年
        var sm = ""; // 月
        var sd = ""; // 日
        var sh = ""; // 時
        var si = ""; // 分
        var ss = ""; // 秒
        var sss = ""; // 秒以下

        if (!string.IsNullOrEmpty(str))
            switch (str.Length)
            {
                case 8:
                    sy = str.Substring(0, 4);
                    sm = str.Substring(4, 2);
                    sd = str.Substring(6, 2);
                    result = string.Format(dateFormat08, sy, sm, sd);
                    break;
                case 14:
                    sy = str.Substring(0, 4);
                    sm = str.Substring(4, 2);
                    sd = str.Substring(6, 2);
                    sh = str.Substring(8, 2);
                    si = str.Substring(10, 2);
                    ss = str.Substring(12, 2);
                    result = string.Format(dateFormat14, sy, sm, sd, sh, si, ss);
                    break;
                case 17:
                    sy = str.Substring(0, 4);
                    sm = str.Substring(4, 2);
                    sd = str.Substring(6, 2);
                    sh = str.Substring(8, 2);
                    si = str.Substring(10, 2);
                    ss = str.Substring(12, 2);
                    sss = str.Substring(14, 3);
                    result = string.Format(dateFormat17, sy, sm, sd, sh, si, ss, sss);
                    break;
                default:
                    result = str;
                    break;
            }

        return result;
    }

    /// <summary>
    ///     IEnumerable&lt;KeyValuePair&lt;string, StringValues&gt;&gt;型のインスタンスをNameValueCollectionに変換する。
    ///     渡されたインスタンスがNullだった場合は空のNameValueCollectionを返す。
    /// </summary>
    /// <param name="collection"></param>
    /// <returns></returns>
    public static NameValueCollection ToNameValueCollection(
        IEnumerable<KeyValuePair<string, StringValues>>? collection)
    {
        var nameValueCollection = new NameValueCollection();
        if (collection == null) return nameValueCollection;
        foreach (var pair in collection) nameValueCollection.Add(pair.Key, pair.Value);

        return nameValueCollection;
    }


    /// <summary>
    ///     POSTリクエストの改ざんチェックを行う
    /// </summary>
    /// <param name="posted">Actionメソッドが受け取ったIFormCollection</param>
    /// <param name="merchantCcId">マーチャントCCID</param>
    /// <param name="merchantPw">マーチャントパスワード</param>
    /// <returns></returns>
    public static bool CheckAuthHash(IFormCollection?
        posted, string merchantCcId, string merchantPw)
    {
        if (posted == null) return false;
        // パラメータ改ざんチェック
        if (!AuthHashUtil.CheckAuthHash(posted, merchantCcId, merchantPw)) return false;

        return true;
    }


    /// <summary>
    ///     GETリクエストの改ざんチェックを行う
    /// </summary>
    /// <param name="queryCollection">Actionメソッドが受け取ったIQueryCollection</param>
    /// <param name="merchantCcId">マーチャントCCID</param>
    /// <param name="merchantPw">マーチャントパスワード</param>
    /// <returns></returns>
    public static bool CheckAuthHash(IQueryCollection? queryCollection, string merchantCcId, string merchantPw)
    {
        // パラメータ改ざんチェック
        if (!AuthHashUtil.CheckAuthHash(queryCollection, merchantCcId, merchantPw)) return false;

        return true;
    }
}